@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -187,7 +187,10 @@ int burn_drive_is_occupied(struct burn_drive *d)
 	if(d->busy == BURN_DRIVE_READING_SYNC ||
 		 d->busy == BURN_DRIVE_WRITING_SYNC)
 		return 2;
-	if(d->busy == BURN_DRIVE_WRITING) {
+	if(d->busy == BURN_DRIVE_WRITING ||
+	   d->busy == BURN_DRIVE_WRITING_LEADIN ||
+	   d->busy == BURN_DRIVE_WRITING_LEADOUT ||
+	   d->busy == BURN_DRIVE_WRITING_PREGAP) {
 
 		/* ts A70928 */
 		/* >>> how do i learn whether the writer thread is still
@@ -340,84 +343,203 @@ int burn_drive_send_default_page_05(struct burn_drive *d, int flag)
 	else
 		burn_write_opts_set_write_type(opts,
 			BURN_WRITE_SAO, BURN_BLOCK_SAO);
-	d->send_write_parameters(d, opts);
+	d->send_write_parameters(d, NULL, -1, opts);
 	burn_write_opts_free(opts);
 	d->sent_default_page_05 = 1;
 	return 1;
 }
 
 
+/* ts A70924 */
+int burn_drive__fd_from_special_adr(char *adr)
+{
+	int fd = -1, i;
+
+	if (strcmp(adr, "-") == 0)
+		fd = 1;
+	if(strncmp(adr, "/dev/fd/", 8) == 0) {
+		for (i = 8; adr[i]; i++)
+			if (!isdigit(adr[i]))
+		break;
+		if (i> 8 && adr[i] == 0)
+			fd = atoi(adr + 8);
+	}
+	return fd;
+}
+
+/* @param flag bit0= accept read-only files and return 2 in this case
+               bit1= accept write-only files and return 3 in this case
+*/
+static int burn_drive__is_rdwr(char *fname, int *stat_ret, 
+                               struct stat *stbuf_ret,
+                               off_t *read_size_ret, int flag)
+{
+	int fd, is_rdwr = 1, ret, getfl_ret, st_ret, mask;
+	struct stat stbuf;
+        off_t read_size = 0;
+
+	memset(&stbuf, 0, sizeof(struct stat));
+	fd = burn_drive__fd_from_special_adr(fname);
+	if (fd >= 0)
+		st_ret = fstat(fd, &stbuf);
+	else
+		st_ret = stat(fname, &stbuf);
+	if (st_ret != -1) {
+		is_rdwr = burn_os_is_2k_seekrw(fname, 0);
+		ret = 1;
+		if (S_ISREG(stbuf.st_mode))
+			read_size = stbuf.st_size;
+		else if (is_rdwr)
+			ret = burn_os_stdio_capacity(fname, &read_size);
+		if (ret <= 0 ||
+		    read_size / (off_t) 2048 >= (off_t) 0x7ffffff0) 
+			read_size = (off_t) 0x7ffffff0 * (off_t) 2048;  
+	}
+
+	if (is_rdwr && fd >= 0) {
+		getfl_ret = fcntl(fd, F_GETFL);
+
+/*
+fprintf(stderr, "LIBBURN_DEBUG: burn_drive__is_rdwr: getfl_ret = %lX , O_RDWR = %lX , & = %lX , O_RDONLY = %lX\n", (unsigned long) getfl_ret, (unsigned long) O_RDWR, (unsigned long) (getfl_ret & O_RDWR), (unsigned long) O_RDONLY);
+*/
+
+		mask = O_RDWR | O_WRONLY | O_RDONLY;
+
+		if (getfl_ret == -1 || (getfl_ret & mask) != O_RDWR)
+			is_rdwr = 0;
+		if ((flag & 1) && getfl_ret != -1 &&
+		    (getfl_ret & mask) == O_RDONLY)
+			is_rdwr = 2;
+		if ((flag & 2) && getfl_ret != -1 &&
+		    (getfl_ret & mask) == O_WRONLY)
+			is_rdwr = 3;
+	}
+	if (stat_ret != NULL)
+		*stat_ret = st_ret;
+	if (stbuf_ret != NULL)
+		memcpy(stbuf_ret, &stbuf, sizeof(struct stat));
+	if (read_size_ret != NULL)
+		*read_size_ret = read_size;
+	return is_rdwr;
+}
+
+
+int burn_drive_grab_stdio(struct burn_drive *d, int flag)
+{
+	int stat_ret = -1, is_rdwr, ret;
+	struct stat stbuf;
+	off_t read_size= 0, size= 0;
+	char fd_name[40], *name_pt = NULL;
+
+	if(d->stdio_fd >= 0) {
+		sprintf(fd_name, "/dev/fd/%d", d->stdio_fd);
+		name_pt = fd_name;
+	} else if (d->devname[0]) {
+		name_pt = d->devname;
+	}
+	if (name_pt != NULL) {
+		/* re-assess d->media_read_capacity and free space */
+		is_rdwr = burn_drive__is_rdwr(name_pt, &stat_ret, &stbuf,
+							 &read_size, 1 | 2);
+		/* despite its name : last valid address, not size */
+		d->media_read_capacity =
+					read_size / 2048 - !(read_size % 2048);
+		if ((stat_ret == -1 || is_rdwr) && d->devname[0]) { 
+		       	ret = burn_os_stdio_capacity(d->devname, &size);
+			if (ret > 0)
+				burn_drive_set_media_capacity_remaining(d,
+									size);
+		}
+	}
+
+	d->released = 0;
+	d->current_profile = 0xffff;
+	if(d->drive_role == 2 || d->drive_role == 3) {
+		d->status = BURN_DISC_BLANK;
+	} else if(d->drive_role == 4) {
+		if (d->media_read_capacity > 0)
+			d->status = BURN_DISC_FULL;
+		else
+			d->status = BURN_DISC_EMPTY;
+	} else if(d->drive_role == 5) {
+		if (stat_ret != -1 && S_ISREG(stbuf.st_mode) &&
+		    stbuf.st_size > 0) {
+			d->status = BURN_DISC_APPENDABLE;
+			if (stbuf.st_size / (off_t) 2048
+			    >= 0x7ffffff0) {
+				d->status = BURN_DISC_FULL;
+				d->role_5_nwa = 0x7ffffff0;
+			} else 
+				d->role_5_nwa = stbuf.st_size / 2048 +
+				              !!(stbuf.st_size % 2048);
+		} else
+			d->status = BURN_DISC_BLANK;
+	} else {
+		d->status = BURN_DISC_EMPTY;
+		d->current_profile = 0;
+	}
+	d->busy = BURN_DRIVE_IDLE;
+	return 1;
+}
+
+
 int burn_drive_grab(struct burn_drive *d, int le)
 {
 	int errcode;
-	/* ts A61125 - B10314 */
-	int ret, sose, stat_ret = -1;
-	struct stat stbuf;
+	/* ts A61125 - B20122 */
+	int ret, sose, signal_action_mem = -1;
 
+	sose = d->silent_on_scsi_error;
 	if (!d->released) {
-		burn_print(1, "can't grab - already grabbed\n");
+                libdax_msgs_submit(libdax_messenger, d->global_index,
+                                0x00020189, LIBDAX_MSGS_SEV_FATAL,
+                                LIBDAX_MSGS_PRIO_LOW,
+				"Drive is already grabbed by libburn", 0, 0);
 		return 0;
 	}
 	if(d->drive_role != 1) {
-		d->released = 0;
-		d->current_profile = 0xffff;
-		if (d->devname[0])
-			stat_ret = stat(d->devname, &stbuf);
-		if(d->drive_role == 2 || d->drive_role == 3) {
-			d->status = BURN_DISC_BLANK;
-		} else if(d->drive_role == 4) {
-			if (d->media_read_capacity > 0)
-				d->status = BURN_DISC_FULL;
-			else
-				d->status = BURN_DISC_EMPTY;
-		} else if(d->drive_role == 5) {
-			if (stat_ret != -1 && S_ISREG(stbuf.st_mode) &&
-			    stbuf.st_size > 0) {
-				d->status = BURN_DISC_APPENDABLE;
-				if (stbuf.st_size / (off_t) 2048
-				    >= 0x7ffffff0) {
-					d->status = BURN_DISC_FULL;
-					d->role_5_nwa = 0x7ffffff0;
-				} else 
-					d->role_5_nwa = stbuf.st_size / 2048 +
-					              !!(stbuf.st_size % 2048);
-			} else
-				d->status = BURN_DISC_BLANK;
-		} else {
-			d->status = BURN_DISC_EMPTY;
-			d->current_profile = 0;
-		}
-		d->busy = BURN_DRIVE_IDLE;
-		return 1;
+		ret = burn_drive_grab_stdio(d, 0);
+		return ret;
 	}
 
 	d->status = BURN_DISC_UNREADY;
 	errcode = d->grab(d);
-
-	if (errcode == 0) {
-		burn_print(1, "low level drive grab failed\n");
+	if (errcode == 0)
 		return 0;
-	}
+
+	burn_grab_prepare_sig_action(&signal_action_mem, 0);
 	d->busy = BURN_DRIVE_GRABBING;
 
 	if (le)
 		d->load(d);
+	if (d->cancel || burn_is_aborting(0))
+		{ret = 0; goto ex;}
 
 	d->lock(d);
+	if (d->cancel || burn_is_aborting(0))
+		{ret = 0; goto ex;}
 
 	/* ts A61118 */
 	d->start_unit(d);
+	if (d->cancel || burn_is_aborting(0))
+		{ret = 0; goto ex;}
 
 	/* ts A61202 : gave bit1 of le a meaning */
-	sose = d->silent_on_scsi_error;
 	if (!le)
 		d->silent_on_scsi_error = 1;
 	/* ts A61125 : outsourced media state inquiry aspects */
 	ret = burn_drive_inquire_media(d);
+	if (d->cancel || burn_is_aborting(0))
+		{ret = 0; goto ex;}
 
 	burn_drive_send_default_page_05(d, 0);
+	if (d->cancel || burn_is_aborting(0))
+		{ret = 0; goto ex;}
+
+ex:;
 	d->silent_on_scsi_error = sose;
 	d->busy = BURN_DRIVE_IDLE;
+	burn_grab_restore_sig_action(signal_action_mem, 0);
 	return ret;
 }
 
@@ -511,7 +633,6 @@ struct burn_drive *burn_drive_finish_enum(struct burn_drive *d)
 	/* try to get the drive info */
 	ret = t->grab(t);
 	if (ret) {
-	        burn_print(2, "getting drive info\n");
 	        t->getcaps(t);
 	        t->unlock(t);
 	        t->released = 1;
@@ -536,7 +657,9 @@ ex:
 
 
 /* ts A61125 : model aspects of burn_drive_release */
-int burn_drive_mark_unready(struct burn_drive *d)
+/* @param flag bit3= do not close d->stdio_fd
+*/
+int burn_drive_mark_unready(struct burn_drive *d, int flag)
 {
 	/* ts A61020 : mark media info as invalid */
 	d->start_lba= -2000000000;
@@ -556,21 +679,23 @@ int burn_drive_mark_unready(struct burn_drive *d)
 		burn_disc_free(d->disc);
 		d->disc = NULL;
 	}
-	if (d->stdio_fd >= 0)
-		close (d->stdio_fd);
-	d->stdio_fd = -1;
+	if (!(flag & 8)) {
+		if (d->stdio_fd >= 0)
+			close (d->stdio_fd);
+		d->stdio_fd = -1;
+	}
 	return 1;
 }
 
 
 /* ts A70918 : outsourced from burn_drive_release() and enhanced */
 /** @param flag bit0-2 = mode : 0=unlock , 1=unlock+eject , 2=leave locked
+                bit3= do not call d->release()
 */
 int burn_drive_release_fl(struct burn_drive *d, int flag)
 {
 	if (d->released) {
 		/* ts A61007 */
-		/* burn_print(1, "second release on drive!\n"); */
 		libdax_msgs_submit(libdax_messenger,
 				d->global_index, 0x00020105,
 				LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
@@ -596,15 +721,17 @@ int burn_drive_release_fl(struct burn_drive *d, int flag)
 			d->unlock(d);
 		if ((flag & 7) == 1)
 			d->eject(d);
-		burn_drive_snooze(d, 0);
-		d->release(d);
+		if (!(flag & 8)) {
+			burn_drive_snooze(d, 0);
+			d->release(d);
+		}
 	}
 
 	d->needs_sync_cache = 0; /* just to be sure */
 	d->released = 1;
 
 	/* ts A61125 : outsourced model aspects */
-	burn_drive_mark_unready(d);
+	burn_drive_mark_unready(d, flag & 8);
 	return 1;
 }
 
@@ -632,6 +759,38 @@ void burn_drive_release(struct burn_drive *d, int le)
 }
 
 
+/* ts B11002 */
+/* API */
+int burn_drive_re_assess(struct burn_drive *d, int flag)
+{
+	int ret, signal_action_mem;
+
+	if (d->released) {
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			     0x00020108,
+			     LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
+			     "Drive is not grabbed on burn_drive_re_assess()",
+			     0, 0);
+		return 0;
+	}
+	burn_drive_release_fl(d, 2 | 8);
+
+	if(d->drive_role != 1) {
+		ret = burn_drive_grab_stdio(d, 0);
+		return ret;
+	}
+
+	burn_grab_prepare_sig_action(&signal_action_mem, 0);
+	d->busy = BURN_DRIVE_GRABBING;
+	ret = burn_drive_inquire_media(d);
+	burn_drive_send_default_page_05(d, 0);
+	d->busy = BURN_DRIVE_IDLE;
+	burn_grab_restore_sig_action(signal_action_mem, 0);
+	d->released = 0;
+	return ret;
+}
+
+
 /* ts A70918 */
 /* API */
 int burn_drive_leave_locked(struct burn_drive *d, int flag)
@@ -687,9 +846,6 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 {
 	int ret;
 
-	burn_print(1, "erasing drive %s %s\n", d->idata->vendor,
-		   d->idata->product);
-
 	if (d->drive_role == 5) { /* Random access write-only drive */
 		ret = truncate(d->devname, (off_t) 0);
 		if (ret == -1) {
@@ -727,16 +883,38 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 	d->erase(d, fast);
 	d->busy = BURN_DRIVE_ERASING;
 
+#ifdef Libburn_old_progress_looP
+
 	/* read the initial 0 stage */
 	while (!d->test_unit_ready(d) && d->get_erase_progress(d) == 0)
 		sleep(1);
 	while ((d->progress.sector = d->get_erase_progress(d)) > 0 ||
 		!d->test_unit_ready(d))
 		sleep(1);
+
+#else /* Libburn_old_progress_looP */
+
+	while (1) {
+		ret = d->get_erase_progress(d);
+		if (ret == -2 || ret > 0)
+	break;
+		sleep(1);
+	}
+	while (1) {
+		ret = d->get_erase_progress(d);
+		if(ret == -2)
+	break;
+		if (ret >= 0)
+			d->progress.sector = ret;
+		sleep(1);
+        }
+
+#endif /* ! Libburn_old_progress_looP */
+
 	d->progress.sector = 0x10000;
 
 	/* ts A61125 : update media state records */
-	burn_drive_mark_unready(d);
+	burn_drive_mark_unready(d, 0);
 	if (d->drive_role == 1)
 		burn_drive_inquire_media(d);
 	d->busy = BURN_DRIVE_IDLE;
@@ -777,6 +955,8 @@ void burn_disc_format_sync(struct burn_drive *d, off_t size, int flag)
 	if (ret <= 0)
 		d->cancel = 1;
 
+#ifdef Libburn_old_progress_looP
+
 	while (!d->test_unit_ready(d) && d->get_erase_progress(d) == 0)
 		sleep(1);
 	while ((pseudo_sector = d->get_erase_progress(d)) > 0 ||
@@ -784,13 +964,33 @@ void burn_disc_format_sync(struct burn_drive *d, off_t size, int flag)
 		d->progress.sector = pseudo_sector / stages;
 		sleep(1);
         }
+
+#else /* Libburn_old_progress_looP */
+
+	while (1) {
+		ret = d->get_erase_progress(d);
+		if (ret == -2 || ret > 0)
+	break;
+		sleep(1);
+	}
+	while (1) {
+		pseudo_sector = d->get_erase_progress(d);
+		if(pseudo_sector == -2)
+	break;
+		if (pseudo_sector >= 0)
+			d->progress.sector = pseudo_sector / stages;
+		sleep(1);
+        }
+
+#endif /* ! Libburn_old_progress_looP */
+
 	d->sync_cache(d);
 
 	if (size <= 0)
 		goto ex;
 
 	/* update media state records */
-	burn_drive_mark_unready(d);
+	burn_drive_mark_unready(d, 0);
 	burn_drive_inquire_media(d);
 	if (flag & 1) {
 		/* write size in zeros */;
@@ -969,17 +1169,6 @@ void burn_drive_cancel(struct burn_drive *d)
 */
 }
 
-/* ts A61007 : defunct because unused */
-#if 0
-int burn_drive_get_block_types(struct burn_drive *d,
-			       enum burn_write_types write_type)
-{
-	burn_print(12, "write type: %d\n", write_type);
-	a ssert(			/* (write_type >= BURN_WRITE_PACKET) && */
-		      (write_type <= BURN_WRITE_RAW));
-	return d->block_types[write_type];
-}
-#endif
 
 static void strip_spaces(char *str)
 {
@@ -1216,7 +1405,7 @@ int burn_drive_forget(struct burn_drive *d, int force)
 	if(occup > 0)
 		if(force < 1)
 			return 0; 
-	if(occup > 10)
+	if(occup >= 10)
 		return 0;
 
 	/* >>> do any drive calming here */;
@@ -1433,80 +1622,6 @@ char *burn_drive_whitelist_item(int idx, int flag)
 }
 
 
-/* ts A70924 */
-int burn_drive__fd_from_special_adr(char *adr)
-{
-	int fd = -1, i;
-
-	if (strcmp(adr, "-") == 0)
-		fd = 1;
-	if(strncmp(adr, "/dev/fd/", 8) == 0) {
-		for (i = 8; adr[i]; i++)
-			if (!isdigit(adr[i]))
-		break;
-		if (i> 8 && adr[i] == 0)
-			fd = atoi(adr + 8);
-	}
-	return fd;
-}
-
-/* @param flag bit0= accept read-only files and return 2 in this case
-               bit1= accept write-only files and return 3 in this case
-*/
-static int burn_drive__is_rdwr(char *fname, int *stat_ret, 
-                               struct stat *stbuf_ret,
-                               off_t *read_size_ret, int flag)
-{
-	int fd, is_rdwr = 1, ret, getfl_ret, st_ret, mask;
-	struct stat stbuf;
-        off_t read_size = 0;
-
-	memset(&stbuf, 0, sizeof(struct stat));
-	fd = burn_drive__fd_from_special_adr(fname);
-	if (fd >= 0)
-		st_ret = fstat(fd, &stbuf);
-	else
-		st_ret = stat(fname, &stbuf);
-	if (st_ret != -1) {
-		is_rdwr = burn_os_is_2k_seekrw(fname, 0);
-		ret = 1;
-		if (S_ISREG(stbuf.st_mode))
-			read_size = stbuf.st_size;
-		else if (is_rdwr)
-			ret = burn_os_stdio_capacity(fname, &read_size);
-		if (ret <= 0 ||
-		    read_size / (off_t) 2048 >= (off_t) 0x7ffffff0) 
-			read_size = (off_t) 0x7ffffff0 * (off_t) 2048;  
-	}
-
-	if (is_rdwr && fd >= 0) {
-		getfl_ret = fcntl(fd, F_GETFL);
-
-/*
-fprintf(stderr, "LIBBURN_DEBUG: burn_drive__is_rdwr: getfl_ret = %lX , O_RDWR = %lX , & = %lX , O_RDONLY = %lX\n", (unsigned long) getfl_ret, (unsigned long) O_RDWR, (unsigned long) (getfl_ret & O_RDWR), (unsigned long) O_RDONLY);
-*/
-
-		mask = O_RDWR | O_WRONLY | O_RDONLY;
-
-		if (getfl_ret == -1 || (getfl_ret & mask) != O_RDWR)
-			is_rdwr = 0;
-		if ((flag & 1) && getfl_ret != -1 &&
-		    (getfl_ret & mask) == O_RDONLY)
-			is_rdwr = 2;
-		if ((flag & 2) && getfl_ret != -1 &&
-		    (getfl_ret & mask) == O_WRONLY)
-			is_rdwr = 3;
-	}
-	if (stat_ret != NULL)
-		*stat_ret = st_ret;
-	if (stbuf_ret != NULL)
-		memcpy(stbuf_ret, &stbuf, sizeof(struct stat));
-	if (read_size_ret != NULL)
-		*read_size_ret = read_size;
-	return is_rdwr;
-}
-
-
 static int burn_role_by_access(char *fname, int flag)
 {
 /* We normally need _LARGEFILE64_SOURCE defined by the build system.
@@ -1726,7 +1841,7 @@ int burn_drive_scan_and_grab(struct burn_drive_info *drive_infos[], char* adr,
 	if (ret < 0)
 		return -1;
 
-	if (n_drives <= 0)
+	if (n_drives == 0)
 		return 0;
 /*
 	fprintf(stderr, "libburn: experimental: n_drives %d , drivetop %d\n",
@@ -2054,8 +2169,9 @@ int burn_drive_convert_fs_adr_sub(char *path, char adr[], int *rec_count)
 	    burn_drive_is_enumerable_adr(path)) {
 		if(strlen(path) >= BURN_DRIVE_ADR_LEN)
 			return -1;
-		burn_drive_adr_debug_msg(
-			"burn_drive_is_enumerable_adr( %s ) is true", path);
+		if (strncmp(path, "stdio:", 6) != 0)
+			burn_drive_adr_debug_msg(
+			   "burn_drive_is_enumerable_adr( %s ) is true", path);
 		strcpy(adr, path);
 		return 1;
 	}
@@ -2265,13 +2381,15 @@ int burn_abort_5(int patience,
 
 			}
 
-			if(occup <= 10) {
+			if(occup < 10) {
+				if (!drive_array[i].cancel)
+					burn_drive_cancel(&(drive_array[i]));
 				if (drive_array[i].drive_role != 1)
 		 			/* occup == -1 comes early */
 					usleep(1000000);
 				burn_drive_forget(&(drive_array[i]), 1);
 			} else if(occup <= 100) {
-				if(first_round)
+				if (!drive_array[i].cancel)
 					burn_drive_cancel(&(drive_array[i]));
 				still_not_done++;
 			} else if(occup <= 1000) {
@@ -2417,7 +2535,7 @@ int burn_disc_track_lba_nwa(struct burn_drive *d, struct burn_write_opts *o,
 	if (d->drive_role != 1)
 		return 0;
 	if (o != NULL)
-		d->send_write_parameters(d, o);
+		d->send_write_parameters(d, NULL, -1, o);
 	ret = d->get_nwa(d, trackno, lba, nwa);
 	return ret;
 }
@@ -2470,7 +2588,7 @@ off_t burn_disc_available_space(struct burn_drive *d,
 			  (off_t) (512 * 1024 * 1024 - 1) * (off_t) 2048);
 	} else {
 		if (o != NULL)
-			d->send_write_parameters(d, o);
+			d->send_write_parameters(d, NULL, -1, o);
 		d->get_nwa(d, -1, &lba, &nwa);
 	}
 	if (o != NULL) {
@@ -2919,11 +3037,12 @@ int burn_disc_get_write_mode_demands(struct burn_disc *disc,
 				last_track_is_unknown = 1;
 			} else
 				last_track_is_unknown = 0;
-			if (mode != track->mode)
+			if ((mode & BURN_MODE_BITS) !=
+						(track->mode & BURN_MODE_BITS))
 				result->mixed_mode = 1;
-			if (track->mode == BURN_MODE1) {
+			if (track->mode & BURN_MODE1) {
 				result->block_types |= BURN_BLOCK_MODE1;
-			} else if (track->mode == BURN_AUDIO) {
+			} else if (track->mode & BURN_AUDIO) {
 				result->audio = 1;
 				result->block_types |= BURN_BLOCK_RAW0;
 				result->exotic_track = 1;
@@ -3212,7 +3331,7 @@ int burn_disc_get_cd_info(struct burn_drive *d, char disc_type[80],
 	}
 	*disc_id = d->disc_id;
 	memcpy(bar_code, d->disc_bar_code, 8);
-	bar_code[9]= 0;
+	bar_code[8]= 0;
 	*app_code = d->disc_app_code;
 	*valid =  d->disc_info_valid;
 	return 1;
@@ -3257,3 +3376,16 @@ int burn_disc_next_track_is_damaged(struct burn_drive *d, int flag)
 }
 
 
+/* ts B11201 : API */
+/* Read the CD-TEXT data from the Lead-in of an Audio CD
+*/
+int burn_disc_get_leadin_text(struct burn_drive *d,
+                              unsigned char **text_packs, int *num_packs,
+                              int flag)
+{
+	int ret;
+
+	ret = mmc_get_leadin_text(d, text_packs, num_packs, 0);
+	return ret;
+}
+
