@@ -1,6 +1,6 @@
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2010 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -46,7 +46,13 @@ struct burn_write_opts *burn_write_opts_new(struct burn_drive *drive)
 	opts->underrun_proof = drive->mdata->underrun_proof;
 	opts->perform_opc = 1;
 	opts->obs = -1;
+
+#ifdef Libburn_dvd_always_obs_paD
+	opts->obs_pad = 1;
+#else
 	opts->obs_pad = 0;
+#endif
+
 	opts->start_byte = -1;
 	opts->fill_up_media = 0;
 	opts->force_is_set = 0;
@@ -480,6 +486,13 @@ void burn_write_opts_set_dvd_obs(struct burn_write_opts *opts, int obs)
 }
 
 
+/* ts B20406: API */
+void burn_write_opts_set_obs_pad(struct burn_write_opts *opts, int pad)
+{
+	opts->obs_pad = 2 * !!pad;
+}
+
+
 /* ts A91115: API */
 void burn_write_opts_set_stdio_fsync(struct burn_write_opts *opts, int rythm)
 {
