@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -73,6 +73,7 @@ int scsi_init_command(struct command *c, unsigned char *opcode, int oplen)
 	c->error = 0;
 	c->retry = 0;
 	c->page = NULL;
+	c->timeout = Libburn_scsi_default_timeouT;
 	return 1;
 }
 
@@ -101,7 +102,8 @@ int spc_decode_sense(unsigned char *sense, int senselen,
 }
 
 
-int spc_test_unit_ready_r(struct burn_drive *d, int *key, int *asc, int *ascq)
+int spc_test_unit_ready_r(struct burn_drive *d, int *key, int *asc, int *ascq,
+				int *progress)
 {
 	struct command *c;
 
@@ -113,8 +115,14 @@ int spc_test_unit_ready_r(struct burn_drive *d, int *key, int *asc, int *ascq)
 	c->retry = 0;
 	c->dir = NO_TRANSFER;
 	d->issue_command(d, c);
+	*key = *asc = *ascq = 0;
+	*progress = -1;
 	if (c->error) {
 		spc_decode_sense(c->sense, 0, key, asc, ascq);
+		if (c->sense[0] == 0x70 &&
+		    ((c->sense[2] & 0x0f) == 0 || (c->sense[2] & 0x0f) == 2) &&
+		    (c->sense[15] & 0x80))
+			*progress = (c->sense[16] << 8) + c->sense[17];
 		return (key == 0);
 	}
 	return 1;
@@ -123,9 +131,9 @@ int spc_test_unit_ready_r(struct burn_drive *d, int *key, int *asc, int *ascq)
 
 int spc_test_unit_ready(struct burn_drive *d)
 {
-	int key,asc,ascq;
+	int key, asc, ascq, progress;
 
-	return spc_test_unit_ready_r(d, &key, &asc, &ascq);
+	return spc_test_unit_ready_r(d, &key, &asc, &ascq, &progress);
 }
 
 
@@ -138,15 +146,21 @@ int spc_wait_unit_attention(struct burn_drive *d, int max_sec, char *cmd_text,
 				int flag)
 {
 	int i, ret = 1, key = 0, asc = 0, ascq = 0, clueless_start = 0;
-	static int clueless_timeout = 5 * 10;
+	static double tests_per_second = 2.0;
+	int sleep_usecs, loop_limit, clueless_timeout, progress;
 	char *msg = NULL;
 	unsigned char sense[14];
 
 	BURN_ALLOC_MEM(msg, char, 320);
+	clueless_timeout = 5 * tests_per_second + 1;
+	loop_limit = max_sec * tests_per_second + 1;
+	sleep_usecs = 1000000 / tests_per_second;
+
 	if (!(flag & 1))
-		usleep(100000);
-	for(i = !(flag & 1); i < max_sec * 10; i++) {
-		ret = spc_test_unit_ready_r(d, &key, &asc, &ascq);
+		usleep(sleep_usecs);
+
+	for(i = !(flag & 1); i < loop_limit; i++) {
+		ret = spc_test_unit_ready_r(d, &key, &asc, &ascq, &progress);
 		if (ret > 0) /* ready */
 	break;
 		if (key!=0x2 || asc!=0x4) {
@@ -202,7 +216,7 @@ handle_error:;
 			goto handle_error;
 
 slumber:;
-		usleep(100000);
+		usleep(sleep_usecs);
 	}
 	if (ret <= 0 || !(flag & 2)) {
 		sprintf(msg, "Async %s %s after %d.%d seconds",
@@ -246,17 +260,37 @@ void spc_request_sense(struct burn_drive *d, struct buffer *buf)
 	d->issue_command(d, c);
 }
 
+/* @return -2 = drive is ready , -1 = not ready, but no progress reported
+           >= 0 progress indication between 0 and 65535
+*/
 int spc_get_erase_progress(struct burn_drive *d)
 {
 	struct buffer *b = NULL;
-	int ret;
+	int ret, key, asc, ascq, progress;
 
 	if (mmc_function_spy(d, "get_erase_progress") <= 0)
 		{ret = 0; goto ex;}
 
+	/* ts B20104 :
+	   TEST UNIT READY seems to be more reliable than REQUEST SENSE.
+	   Nevertheless growisofs still uses the latter as fallback.
+	*/
+	ret = spc_test_unit_ready_r(d, &key, &asc, &ascq, &progress);
+	if (ret > 0)
+		{ret = -2; goto ex;}
+	if (progress >= 0)
+		{ret = progress; goto ex;}
+
+	/* Fallback to request sense */
 	BURN_ALLOC_MEM(b, struct buffer, 1);
 	spc_request_sense(d, b);
-	ret = (b->data[16] << 8) | b->data[17];
+
+	/* Checking the preconditions as of SPC-3 4.5.2.4.4 and 4.5.3 */
+	ret = -1;
+	if (b->data[0] == 0x70 &&
+	    ((b->data[2] & 0x0f) == 0 || (b->data[2] & 0x0f) == 2) &&
+	    (b->data[15] & 0x80))
+		ret = (b->data[16] << 8) | b->data[17];
 ex:;
 	BURN_FREE_MEM(b);
 	return ret;
@@ -341,7 +375,7 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	struct buffer *buf = NULL;
 	struct scsi_mode_data *m;
 	int page_length, num_write_speeds = 0, i, speed, ret;
-	int old_alloc_len, was_error = 0;
+	int old_alloc_len, was_error = 0, block_descr_len;
 	unsigned char *page;
 	struct command *c = NULL;
 	struct burn_speed_descriptor *sd;
@@ -364,10 +398,6 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 
 	memset(buf, 0, sizeof(struct buffer));
 	scsi_init_command(c, SPC_MODE_SENSE, sizeof(SPC_MODE_SENSE));
-/*
-	memcpy(c->opcode, SPC_MODE_SENSE, sizeof(SPC_MODE_SENSE));
-	c->oplen = sizeof(SPC_MODE_SENSE);
-*/
 	c->dxfer_len = *alloc_len;
 	c->opcode[7] = (c->dxfer_len >> 8) & 0xff;
 	c->opcode[8] = c->dxfer_len & 0xff;
@@ -384,13 +414,22 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 		was_error = 1;
 	}
 
-	page = c->page->data + 8;
+	/* ts B11103 : qemu SCSI CD-ROM has Block Descriptor Length > 0.
+	               The descriptors come between header and page.
+	*/
+	block_descr_len = c->page->data[6] * 256 + c->page->data[7];
+
+	/* Skip over Mode Data Header and block descriptors */
+	page = c->page->data + 8 + block_descr_len;
 
 	/* ts A61225 :
 	   Although MODE SENSE indeed belongs to SPC, the returned code page
 	   2Ah is part of MMC-1 to MMC-3. In MMC-1 5.2.3.4. it has 22 bytes,
 	   in MMC-3 6.3.11 there are at least 28 bytes plus a variable length
 	   set of speed descriptors. In MMC-5 E.11 it is declared "legacy".
+	   ts B11031 :
+	   qemu emulates an ATAPI DVD-ROM, which delivers only a page length
+	   of 18. This is now tolerated.
 	*/
 	/* ts A90603 :
 	   SPC-1 8.3.3 enumerates mode page format bytes from 0 to n and
@@ -404,8 +443,9 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	if (page_length + 10 > old_alloc_len)
 		page_length = old_alloc_len - 10;
 
-	/* ts A90602 : 20 asserts page[21]. (see SPC-1 8.3.3) */
-	if (page_length < 20) {
+	/* ts A90602 : page_length N asserts page[N+1]. (see SPC-1 8.3.3) */
+	/* ts B11031 : qemu drive has a page_length of 18 */
+	if (page_length < 18) {
 		m->valid = -1;
 		sprintf(msg, "MODE SENSE page 2A too short: %s : %d",
 			d->devname, page_length);
@@ -434,8 +474,11 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	m->max_read_speed = page[8] * 256 + page[9];
 	m->cur_read_speed = page[14] * 256 + page[15];
 
-	m->max_write_speed = page[18] * 256 + page[19];
-	m->cur_write_speed = page[20] * 256 + page[21];
+	m->max_write_speed = m->cur_write_speed = 0;
+	if (page_length >= 18) /* note: page length is page size - 2 */
+		m->max_write_speed = page[18] * 256 + page[19];
+	if (page_length >= 20)
+		m->cur_write_speed = page[20] * 256 + page[21];
 
 	/* ts A61021 : New field to be set by atip (or following MMC-3 info) */
 	m->min_write_speed = m->max_write_speed;
@@ -535,7 +578,7 @@ ex:
 
 void spc_sense_caps(struct burn_drive *d)
 {
-	int alloc_len, start_len = 30, ret;
+	int alloc_len, start_len = 30, minimum_len = 28, ret;
 
 	mmc_start_if_needed(d, 1);
 	if (mmc_function_spy(d, "sense_caps") <= 0)
@@ -548,7 +591,12 @@ void spc_sense_caps(struct burn_drive *d)
 	fprintf(stderr,"LIBBURN_DEBUG: 5Ah alloc_len = %d , ret = %d\n",
 			alloc_len, ret);
 */
-	if (alloc_len >= start_len && ret > 0)
+	/* ts B11103:
+	   qemu ATAPI DVD-ROM delivers only 28.
+	   SanDisk Cruzer U3 memory stick throws error on alloc_len < 30.
+	   MMC-1 prescribes that 30 are available. qemu tolerates 30.
+	*/
+	if (alloc_len >= minimum_len && ret > 0)
 		/* second execution with announced length */
 		spc_sense_caps_al(d, &alloc_len, 0);
 }
@@ -621,7 +669,6 @@ void spc_select_error_params(struct burn_drive *d,
 		c->page->data[10] |= 4;
 	if (!o->hardware_error_recovery)
 		c->page->data[10] |= 1;
-/*burn_print(1, "error parameter 0x%x\n", c->page->data[10]);*/
 	c->page->data[11] = d->params.retries;
 	c->dir = TO_DRIVE;
 	d->issue_command(d, c);
@@ -665,7 +712,6 @@ void spc_sense_write_params(struct burn_drive *d)
 	m = d->mdata;
 	if (!c->error) {
 		page = c->page->data + 8;
-		burn_print(1, "write page length 0x%x\n", page[1]);
 		m->write_page_length = page[1];
 		m->write_page_valid = 1;
 	} else
@@ -699,8 +745,8 @@ Although command MODE SELECT is SPC, the content of the
 Write Parameters Mode Page (05h) is MMC (Table 108 in MMC-1). 
 Thus the filling of the mode page is done by mmc_compose_mode_page_5().
 */
-void spc_select_write_params(struct burn_drive *d,
-			     const struct burn_write_opts *o)
+void spc_select_write_params(struct burn_drive *d, struct burn_session *s,
+			     int tnum, const struct burn_write_opts *o)
 {
 	struct buffer *buf = NULL;
 	struct command *c = NULL;
@@ -756,11 +802,8 @@ void spc_select_write_params(struct burn_drive *d,
 
 	c->page->bytes = alloc_len;
 
-	burn_print(12, "using write page length %d (valid %d)\n",
-		   d->mdata->write_page_length, d->mdata->write_page_valid);
-
 	/* ts A61229 */
-	if (mmc_compose_mode_page_5(d, o, c->page->data + 8) <= 0)
+	if (mmc_compose_mode_page_5(d, s, tnum, o, c->page->data + 8) <= 0)
 		goto ex;
 
 	c->dir = TO_DRIVE;
@@ -804,9 +847,6 @@ void spc_probe_write_modes(struct burn_drive *d)
 
 	/* ts A70213 : added pseudo try_write_type 4 to set a suitable mode */
 	while (try_write_type != 5) {
-		burn_print(9, "trying %d, %d\n", try_write_type,
-			   try_block_type);
-
 		/* ts A70213 */
 		if (try_write_type == 4) {
 			/* Pseudo write type NONE . Set a useable write mode */
@@ -845,10 +885,9 @@ void spc_probe_write_modes(struct burn_drive *d)
 
 		spc_decode_sense(c->sense, 0, &key, &asc, &ascq);
 		if (key)
-			burn_print(7, "%d not supported\n", try_block_type);
+			/* try_block_type not supported */;
 		else {
-			burn_print(7, "%d:%d SUPPORTED MODE!\n",
-				   try_write_type, try_block_type);
+			/* try_write_type, try_block_type is supported mode */
 			if (try_write_type == 2)	/* sao */
 				d->block_types[try_write_type] =
 					BURN_BLOCK_SAO;
@@ -1029,8 +1068,10 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 	sprintf(msg, "[%X %2.2X %2.2X] ", *key, *asc, *ascq);
 	msg= msg + strlen(msg);
 
-	burn_print(12, "CONDITION: 0x%x 0x%x 0x%x on %s %s\n",
-		   *key, *asc, *ascq, d->idata->vendor, d->idata->product);
+	if (key_def[*key & 0xf][0] != '(') {
+		sprintf(msg, "%s. ", key_def[*key & 0xf]);
+		msg= msg + strlen(msg);
+	}
 
 	switch (*asc) {
 	case 0x00:
@@ -1041,17 +1082,21 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 
 	case 0x02:
 		sprintf(msg, "Not ready");
-		return RETRY;
+		goto return_retry;
 	case 0x04:
 		if (*ascq == 1)
 			sprintf(msg,
 			"Logical unit is in the process of becoming ready");
 		else
 			sprintf(msg, "Logical unit is not ready");
-		return RETRY;
-	case 0x08:
-		if (*key != 4)
+		goto return_retry;
+	case 0x06:
+		if (*ascq == 0)
+			sprintf(msg, "No reference position found");
+		else
 			break;
+		goto return_fail;
+	case 0x08:
 		if (*ascq == 0)
 			sprintf(msg, "Logical unit communication failure");
 		else if (*ascq == 1)
@@ -1060,10 +1105,10 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 			sprintf(msg, "Logical unit communication parity error");
 		else if (*ascq == 3)
 			sprintf(msg, "Logical unit communication crc error");
-		return RETRY;
-	case 0x09:
-		if (*key != 4)
+		else
 			break;
+		goto return_retry;
+	case 0x09:
 		if (*ascq == 0)
 			sprintf(msg, "Track following error");
 		else if (*ascq == 1)
@@ -1076,24 +1121,27 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 			sprintf(msg, "Head select fault");
 		else
 			break;
-		return FAIL;
+		goto return_fail;
 	case 0x0C:
-		if (*key == 2 && *ascq == 7)
-			sprintf(msg, "Write error, recovery needed"); 
-		else if (*key == 2 && *ascq == 0x0f)
-			sprintf(msg, "Defects in error window"); 
-		else if (*key == 3 && *ascq == 2)
-			sprintf(msg, "Write error, auto reallocation failed");
-		else if (*key == 3 && *ascq == 9)
-			sprintf(msg, "Write error, loss of streaming");
-		else if (*key == 3)
+		if (*ascq == 0)
 			sprintf(msg, "Write error");
+		else if (*ascq == 1)
+			sprintf(msg,
+				"Write error, recovered with auto-allocation");
+		else if (*ascq == 2)
+			sprintf(msg, "Write error, auto reallocation failed");
+		else if (*ascq == 7)
+			sprintf(msg, "Write error, recovery needed"); 
+		else if (*ascq == 8)
+			sprintf(msg, "Write error, recovery failed"); 
+		else if (*ascq == 9)
+			sprintf(msg, "Write error, loss of streaming");
+		else if (*ascq == 0x0f)
+			sprintf(msg, "Defects in error window"); 
 		else
 			break;
-		return FAIL;
+		goto return_fail;
 	case 0x11:
-		if (*key != 3)
-			break;
 		if (*ascq == 0)
 			sprintf(msg, "Unrecovered read error");
 		else if (*ascq == 1)
@@ -1106,70 +1154,73 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 			sprintf(msg, "CIRC uncorrectable error");
 		else
 			break;
-		return FAIL;
+		goto return_fail;
 	case 0x15:
-		if (*key != 3 && *key != 4)
+		if (*ascq == 0)
+			sprintf(msg, "Random positioning error");
+		else if (*ascq == 1)
+			sprintf(msg, "Mechanical positioning error");
+		else
 			break;
-		sprintf(msg, "Random positioning error");
-		return FAIL;
+		goto return_fail;
 	case 0x1a:
-		if (*key != 5)
+		if (*ascq == 0)
+			sprintf(msg, "Parameter list length error");
+		else
 			break;
-		sprintf(msg, "Parameter list length error");
-		return FAIL;
+		goto return_fail;
 	case 0x1b:
-		if (*key != 4)
+		if (*ascq == 0)
+			sprintf(msg, "Synchronous data transfer error");
+		else
 			break;
-		sprintf(msg, "Synchronous data transfer error");
-		return FAIL;
+		goto return_fail;
 	case 0x20:
-		if (*key != 5)
+		if (*ascq == 0)
+			sprintf(msg, "Invalid command operation code");
+		else
 			break;
-		sprintf(msg, "Invalid command operation code");
-		return FAIL;
+		goto return_fail;
 	case 0x21:
-		if (*key != 5)
-			break;
 		if (*ascq == 0)
 			sprintf(msg, "Lba out of range");
+		else if (*ascq == 1)
+			sprintf(msg, "Invalid element address");
+		else if (*ascq == 2)
+			sprintf(msg, "Invalid address for write");
 		else if (*ascq == 3)
 			sprintf(msg, "Invalid write crossing layer jump");
 		else
-			sprintf(msg, "Invalid address");
-		return FAIL;
+			break;
+		goto return_fail;
 	case 0x24:
-		if (*key != 5)
+		if (*ascq == 0)
+			sprintf(msg, "Invalid field in cdb");
+		else
 			break;
-		sprintf(msg, "Invalid field in cdb");
-		return FAIL;
+		goto return_fail;
 	case 0x26:
-		if (*key != 5)
-			break;
-		if (*ascq == 1)
+		if (*ascq == 0)
+			sprintf(msg, "Invalid field in parameter list");
+		else if (*ascq == 1)
 			sprintf(msg, "Parameter not supported");
 		else if (*ascq == 2)
 			sprintf(msg, "Parameter value invalid");
 		else
-			sprintf(msg, "Invalid field in parameter list");
-		return FAIL;
+			break;
+		goto return_fail;
 	case 0x27:
-		if (*key != 7)
-			break;
 		sprintf(msg, "Write protected");
-		return FAIL;
+		goto return_fail;
 	case 0x28:
-		if (*key != 6)
-			break;
 		if (*ascq == 0)
 			sprintf(msg, "Medium may have changed");
 		else if (*ascq == 2)
 			sprintf(msg, "Format layer may have changed");
 		else
 			break;
-		return RETRY;
+		goto return_retry;
 	case 0x29:
-		if (*key != 6)
-			break;
 		if (*ascq == 0)
 			sprintf(msg,
                                "Power on, reset, or bus device reset occured");
@@ -1183,28 +1234,23 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 			sprintf(msg, "Device internal reset");
 		else
 			break;
-		return RETRY;
+		goto return_retry;
 	case 0x2c:
-		if (*key != 5)
-			break;
 		if (*ascq == 0)
 			sprintf(msg, "Command sequence error");
 		else 
 			break;
-		return FAIL;
+		goto return_fail;
 	case 0x2e:
-		if (*key != 6)
-			break;
 		if (*ascq == 0)
-			sprintf(msg,
-                               "Insufficient time for operation");
+			sprintf(msg, "Insufficient time for operation");
 		else 
 			break;
-		return FAIL;
+		goto return_fail;
 	case 0x30:
-		if (*key != 2 && *key != 5)
-			break;
-		if (*ascq == 1)
+		if (*ascq == 0)
+			sprintf(msg, "Incompatible medium installed");
+		else if (*ascq == 1)
 			sprintf(msg, "Cannot read medium, unknown format");
 		else if (*ascq == 2)
 			sprintf(msg,
@@ -1220,47 +1266,50 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 		else if (*ascq == 7)
 			sprintf(msg, "Cleaning failure");
 		else
-			sprintf(msg, "Incompatible medium installed");
-		return FAIL;
-	case 0x31:
-		if (*key != 3)
 			break;
+		goto return_fail;
+	case 0x31:
 		if (*ascq == 0)
 			sprintf(msg, "Medium unformatted or format corrupted");
 		else if (*ascq == 1)
 			sprintf(msg, "Format command failed");
-		return FAIL;
-	case 0x3A:
-		if (*key != 2)
+		else
 			break;
-		if (*ascq == 1)
+		goto return_fail;
+	case 0x3A:
+		if (*ascq == 0)
+			sprintf(msg, "Medium not present");
+		else if (*ascq == 1)
 			sprintf(msg, "Medium not present, tray closed");
 		else if (*ascq == 2)
 			sprintf(msg, "Medium not present, tray open");
 		else if (*ascq == 3)
 			sprintf(msg, "Medium not present, loadable");
 		else
-			sprintf(msg, "Medium not present");
+			break;
 		d->status = BURN_DISC_EMPTY;
-		return FAIL;
+		goto return_fail;
 	case 0x3E:
 		if (*ascq == 1)
 			sprintf(msg, "Logical unit failure");
 		else if (*ascq == 2)
 			sprintf(msg, "Timeout on logical unit");
-		return FAIL;
+		else
+			break;
+		goto return_fail;
 	case 0x44:
 		if (*ascq == 0)
 			sprintf(msg, "Internal target failure");
-		return FAIL;
+		else
+			break;
+		goto return_fail;
 	case 0x57:
-		if (*key != 3 || *ascq != 0)
+		if (*ascq == 0)
+			sprintf(msg, "Unable to recover Table-of-Content");
+		else
 			break;
-		sprintf(msg, "Unable to recover Table-of-Content");
-		return FAIL;
+		goto return_fail;
 	case 0x63:
-		if (*key != 5)
-			break;
 		if (*ascq == 0)
 			sprintf(msg,
 				"End of user area encountered on this track");
@@ -1268,51 +1317,67 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 			sprintf(msg, "Packet does not fit in available space");
 		else
 			break;
-		return FAIL;
+		goto return_fail;
 	case 0x64:
-		if (*key != 5)
-			break;
 		if (*ascq == 0)
 			sprintf(msg, "Illegal mode for this track");
 		else if (*ascq == 1)
 			sprintf(msg, "Invalid packet size");
 		else
 			break;
-		return FAIL;
+		goto return_fail;
 	case 0x72:
-		if (*key == 3)
+		if (*ascq == 0)
 			sprintf(msg, "Session fixation error");
-		else if (*key == 5 && *ascq == 3)
+		else if (*ascq == 1)
+			sprintf(msg, "Session fixation error writing lead-in");
+		else if (*ascq == 2)
+			sprintf(msg,
+				"Session fixation error writing lead-out");
+		else if (*ascq == 3)
 			sprintf(msg,
 			"Session fixation error, incomplete track in session");
-		else if (*key == 5 && *ascq == 4)
+		else if (*ascq == 4)
 			sprintf(msg,
-			"Empty or partially written reserved track");
-		else if (*key == 5 && *ascq == 5)
-			sprintf(msg,
-			"No more track reservations allowed");
+				"Empty or partially written reserved track");
+		else if (*ascq == 5)
+			sprintf(msg, "No more track reservations allowed");
 		else
 			break;
-		return FAIL;
+		goto return_fail;
 	case 0x73:
-		if (*key == 3 && *ascq == 0)
+		if (*ascq == 0)
 			sprintf(msg, "CD control error");
-		else if (*key == 3 && *ascq == 2)
+		else if (*ascq == 1)
+			sprintf(msg, "Power calibration area almost full");
+		else if (*ascq == 2)
 			sprintf(msg, "Power calibration area is full");
-		else if (*key == 3 && *ascq == 3)
+		else if (*ascq == 3)
 			sprintf(msg, "Power calibration area error");
-		else if (*key == 3 && *ascq == 4)
+		else if (*ascq == 4)
 			sprintf(msg, "Program memory area update failure");
-		else if (*key == 3 && *ascq == 5)
+		else if (*ascq == 5)
 			sprintf(msg, "Program memory area is full");
 		else
 			break;
-		return FAIL;
+		goto return_fail;
 	}
 	sprintf(msg_data,
 		"See MMC specs: Sense Key %X \"%s\", ASC %2.2X ASCQ %2.2X",
 		*key & 0xf, key_def[(*key) & 0xf], *asc, *ascq);
+	goto return_fail;
+
+return_fail:
+	strcat(msg, ".");
+	if (*key == 1)
+		return GO_ON;
 	return FAIL;
+
+return_retry:;
+	strcat(msg, ".");
+	if (*key == 1)
+		return GO_ON;
+	return RETRY;
 }
 
 
@@ -1327,10 +1392,6 @@ enum response scsi_error(struct burn_drive *d, unsigned char *sense,
 
 	BURN_ALLOC_MEM(msg, char, 160);
 	resp = scsi_error_msg(d, sense, senselen, msg, &key, &asc, &ascq);
-	if (asc == 0 || asc == 0x3A)
-		burn_print(12, "%s\n", msg);
-	else
-		burn_print(1, "%s\n", msg);
 ex:;
 	if (ret == -1)
 		resp = FAIL;
@@ -1403,7 +1464,7 @@ static char *scsi_command_name(unsigned int c, int flag)
         case 0xbe:
 		return "READ CD";
 	}
-	return "(NOT IN COMMAND LIST)";
+	return "(NOT IN LIBBURN COMMAND LIST)";
 }
 
 
@@ -1451,38 +1512,39 @@ ex:;
 }
 
 
-/* ts A91106 */
+/* ts B11110 */
 /* @param flag bit0= do not show eventual data payload sent to the drive
                      (never with WRITE commands)
-               bit1= show write length and target LBA in decimal
 */
-int scsi_show_cmd_text(struct command *c, void *fp_in, int flag)
+int scsi_show_command(unsigned char *opcode, int oplen, int dir,
+                      unsigned char *data, int bytes,
+                      void *fp_in, int flag)
 {
 	int i;
 	FILE *fp = fp_in;
 
 	fprintf(fp, "\n%s\n",
-		 scsi_command_name((unsigned int) c->opcode[0], 0));
-	for(i = 0; i < 16 && i < c->oplen; i++)
-		fprintf(fp, "%2.2x ", c->opcode[i]);
+		 scsi_command_name((unsigned int) opcode[0], 0));
+	for(i = 0; i < 16 && i < oplen; i++)
+		fprintf(fp, "%2.2x ", opcode[i]);
 	if (i > 0)
 		fprintf(fp, "\n");
 	if (flag & 1)
 		return 1;
-	if (c->opcode[0] == 0x2A) { /* WRITE 10 */
+	if (opcode[0] == 0x2A) { /* WRITE 10 */
 		if (flag & 2)
 			fprintf(fp, "%d -> %d\n",
-				(c->opcode[7] << 8) | c->opcode[8], 
-				mmc_four_char_to_int(c->opcode + 2));
-	} else if (c->opcode[0] == 0xAA) { /* WRITE 12 */
+				(opcode[7] << 8) | opcode[8], 
+				mmc_four_char_to_int(opcode + 2));
+	} else if (opcode[0] == 0xAA) { /* WRITE 12 */
 		if (flag & 2)
 			fprintf(fp, "%d -> %d\n",
-				mmc_four_char_to_int(c->opcode + 6),
-				mmc_four_char_to_int(c->opcode + 2));	
-	} else if (c->dir == TO_DRIVE) {
-		fprintf(fp, "To drive: %db\n", c->page->bytes);
-		for (i = 0; i < c->page->bytes; i++) 
-			fprintf(fp, "%2.2x%c", c->page->data[i],
+				mmc_four_char_to_int(opcode + 6),
+				mmc_four_char_to_int(opcode + 2));	
+	} else if (dir == TO_DRIVE && !(flag & 1)) {
+		fprintf(fp, "To drive: %db\n", bytes);
+		for (i = 0; i < bytes; i++) 
+			fprintf(fp, "%2.2x%c", data[i],
 				((i % 20) == 19 ? '\n' : ' '));
 		if (i % 20)
 			fprintf(fp, "\n");
@@ -1490,24 +1552,39 @@ int scsi_show_cmd_text(struct command *c, void *fp_in, int flag)
 	return 1;
 }
 
+
+
 /* ts A91106 */
-int scsi_show_cmd_reply(struct command *c, void *fp_in, int flag)
+/* @param flag bit0= do not show eventual data payload sent to the drive
+                     (never with WRITE commands)
+*/
+int scsi_show_cmd_text(struct command *c, void *fp_in, int flag)
+{
+	return scsi_show_command(c->opcode, c->oplen, c->dir, c->page->data,
+	                         c->page->bytes, fp_in, flag);
+}
+
+
+/* ts A91106 */ /* ts B11110 */
+int scsi_show_command_reply(unsigned char *opcode, int data_dir,
+                            unsigned char *data, int dxfer_len,
+                            void *fp_in, int flag)
 {
 	int i;
 	FILE *fp = fp_in;
 
-	if (c->dir != FROM_DRIVE)
+	if (data_dir != FROM_DRIVE)
 		return 2;
-	if (c->opcode[0] == 0x28 || c->opcode[0] == 0x3C ||
-	    c->opcode[0] == 0xA8 || c->opcode[0] == 0xBE) {
+	if (opcode[0] == 0x28 || opcode[0] == 0x3C ||
+	    opcode[0] == 0xA8 || opcode[0] == 0xBE) {
 							/* READ commands */
 		/* >>> report amount of data */;
 
 		return 2;
 	}
-	fprintf(fp, "From drive: %db\n", c->dxfer_len);
-	for (i = 0; i < c->dxfer_len; i++)
-		fprintf(fp, "%2.2x%c", c->page->data[i],
+	fprintf(fp, "From drive: %db\n", dxfer_len);
+	for (i = 0; i < dxfer_len; i++)
+		fprintf(fp, "%2.2x%c", data[i],
 			((i % 20) == 19 ? '\n' : ' '));
 	if (i % 20)
 		fprintf(fp, "\n");
@@ -1515,31 +1592,51 @@ int scsi_show_cmd_reply(struct command *c, void *fp_in, int flag)
 }
 
 
+/* ts B11110 */ 
+/** Logs command (before execution) */
+int scsi_log_command(unsigned char *opcode, int oplen, int data_dir,
+                     unsigned char *data, int bytes,
+                     void *fp_in, int flag)
+{
+	FILE *fp = fp_in;
+
+	if (fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
+		scsi_show_command(opcode, oplen, data_dir, data, bytes, fp, 0);
+		if (burn_sg_log_scsi & 4)
+			fflush(fp);
+	}
+	if (fp == stderr || !(burn_sg_log_scsi & 2))
+		return 1;
+	scsi_log_command(opcode, oplen, data_dir, data, bytes, stderr, 0);
+	return 1;
+}
+
+
 /* ts A91218 (former sg_log_cmd ts A70518) */ 
 /** Logs command (before execution) */
 int scsi_log_cmd(struct command *c, void *fp_in, int flag)
 {
-	FILE *fp = fp_in;
+	int ret, bytes = 0;
+	unsigned char *data = NULL;
 
-	if (fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
-		scsi_show_cmd_text(c, fp, 0);
-		if (burn_sg_log_scsi & 4)
-			fflush(fp);
+	if (c->page != NULL) {
+		data = c->page->data;
+		bytes = c->page->bytes;
 	}
-	if (fp == stderr || !(burn_sg_log_scsi & 2))
-		return 1;
-	scsi_log_cmd(c, stderr, flag);
-	return 1;
+	ret = scsi_log_command(c->opcode, c->oplen, c->dir, data, bytes,
+	                       fp_in, flag);
+	return ret;
 }
 
 
-/* ts A91221 (former sg_log_err ts A91108) */
+/* ts B11110 */
 /** Logs outcome of a sg command.
     @param flag  bit0 causes an error message
                  bit1 do not print duration
 */
-int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
-		 int sense_len, int duration, int flag)
+int scsi_log_reply(unsigned char *opcode, int data_dir, unsigned char *data,
+                   int dxfer_len, void *fp_in, unsigned char sense[18],
+		   int sense_len, int duration, int flag)
 {
 	char durtxt[20];
 	FILE *fp = fp_in;
@@ -1565,7 +1662,9 @@ int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
 				(unsigned int) key, (unsigned int) asc,
 				(unsigned int) ascq, durtxt);
 		} else {
-			scsi_show_cmd_reply(c, fp, 0);
+			scsi_show_command_reply(opcode, data_dir, data,
+			                        dxfer_len, fp, 0);
+
 			if (!(flag & 2))
 				fprintf(fp,"%6d ms\n", duration);
 		}
@@ -1574,11 +1673,32 @@ int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
 	}
 	if (fp == stderr || !(burn_sg_log_scsi & 2))
 		return 1;
-	scsi_log_err(c, stderr, sense, sense_len, duration, flag);
+	scsi_log_reply(opcode, data_dir, data, dxfer_len,
+	               stderr, sense, sense_len, duration, flag);
+
 	return 1;
 }
 
 
+/* ts A91221 (former sg_log_err ts A91108) */
+/** Legacy frontend to scsi_log_reply().
+    @param flag  bit0 causes an error message
+                 bit1 do not print duration
+*/
+int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
+		 int sense_len, int duration, int flag)
+{
+	int ret;
+	unsigned char *data = NULL;
+
+	if (c->page != NULL)
+		data = c->page->data;
+	ret= scsi_log_reply(c->opcode, c->dir, data, c->dxfer_len ,
+	                    fp_in, sense, sense_len, duration, flag);
+	return ret;
+}
+
+
 /* ts B00808 */
 /*
     @param flag    bit0 = do not retry
@@ -1592,37 +1712,64 @@ int scsi_eval_cmd_outcome(struct burn_drive *d, struct command *c, void *fp,
 {
 	enum response outcome;
 	int done = -1, usleep_time;
+	char *msg = NULL;
 
 	if (burn_sg_log_scsi & 3)
 		scsi_log_err(c, fp, sense, sense_len, duration,
 				 (sense_len > 0) | (flag & 2));
 	if (sense_len <= 0)
-		return 1;
+		{done = 1; goto ex;}
 		
 	outcome = scsi_error(d, sense, sense_len);
 	if (outcome == RETRY && c->retry && !(flag & 1)) {
 		/* Calming down retries and breaking up endless cycle
 		*/
-		usleep_time = Libburn_scsi_retry_usleeP +
-				loop_count * Libburn_scsi_retry_incR;
+		if (c->opcode[0] == 0x2A || c->opcode[0] == 0xAA) {
+			/* WRITE(10) , WRITE(12) */
+			usleep_time = Libburn_scsi_write_retry_usleeP +
+				loop_count * Libburn_scsi_write_retry_incR;
+			if (usleep_time > Libburn_scsi_write_retry_umaX)
+				usleep_time = Libburn_scsi_write_retry_umaX;
+		} else {
+			usleep_time = Libburn_scsi_retry_usleeP +
+					loop_count * Libburn_scsi_retry_incR;
+			if (usleep_time > Libburn_scsi_retry_umaX)
+				usleep_time = Libburn_scsi_retry_umaX;
+		}
 		if (time(NULL) + usleep_time / 1000000 - start_time >
 		    timeout_ms / 1000 + 1) {
+			done = -1; /* In case of alloc failure */
+			BURN_ALLOC_MEM_VOID(msg, char, 320);
 			done = 1;
-			goto ex;
+			sprintf(msg,
+				"Timeout exceed (%d ms). Retry canceled.\n",
+				timeout_ms);
+			libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x0002018a,
+				LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
+				msg, 0, 0);
+			goto err_ex;
 		}
-		usleep(usleep_time);
+		if (d->cancel)
+			{done = 1; goto ex;}
+		if (usleep_time > 0)
+			usleep(usleep_time);
+		if (d->cancel)
+			{done = 1; goto ex;}
 		if (burn_sg_log_scsi & 3) 
 			scsi_log_cmd(c, fp, 0);
-		return 0;
+		{done = 0; goto ex;}
 	} else if (outcome == RETRY) {
 		done = 1;
 	} else if (outcome == GO_ON) {
-		return 1;
+		{done = 1; goto ex;}
 	} else if (outcome == FAIL) {
 		done = 1;
 	}
-ex:;
+err_ex:;
 	c->error = 1;
 	scsi_notify_error(d, c, sense, sense_len, 0);
+ex:;
+	BURN_FREE_MEM(msg);
 	return done;
 }
