@@ -70,6 +70,23 @@
 #endif
 
 
+/* The theoretical maximum number of Apple Partition Map entries in the
+   System Area of an ISO image:
+   Block0 plus 63 entries with block size 512
+*/
+#define ISO_APM_ENTRIES_MAX 63
+
+/* The maximum number of MBR partition table entries.
+*/
+#define ISO_MBR_ENTRIES_MAX  4
+
+/* The theoretical maximum number of GPT entries in the System Area of an
+   ISO image:
+   MBR plus GPT header block plus 248 GPT entries of 128 bytes each. 
+*/
+#define ISO_GPT_ENTRIES_MAX 248
+
+
 /**
  * Holds the options for the image generation.
  */
@@ -83,6 +100,8 @@ struct iso_write_opts {
     unsigned int rockridge :1;
     unsigned int joliet :1;
     unsigned int iso1999 :1;
+    unsigned int hfsplus :1;
+    unsigned int fat :1;
 
     unsigned int aaip :1; /* whether to write eventual ACL and EAs */
 
@@ -433,6 +452,16 @@ struct iso_write_opts {
     */
     uint32_t tail_blocks;
 
+    /* Eventual disk file path of a PreP partition which shall be prepended
+       to HFS+/FAT and IsoFileSrc areas and marked by an MBR partition entry.
+    */
+    char *prep_partition;
+
+    /* Eventual disk file path of an EFI system partition image which shall
+       be prepended to HFS+/FAT and IsoFileSrc areas and marked by a GPT entry.
+    */
+    char *efi_boot_partition;
+
     /* Eventual disk file paths of prepared images which shall be appended
        after the ISO image and described by partiton table entries in a MBR
     */
@@ -442,12 +471,27 @@ struct iso_write_opts {
     /* Eventual name of the non-ISO aspect of the image. E.g. SUN ASCII label.
      */
     char ascii_disc_label[ISO_DISC_LABEL_SIZE];
+
+    /* HFS+ image serial number.
+     * 00...00 means that it shall be generated by libisofs.
+     */
+    uint8_t hfsp_serial_number[8];
+
+    /* Allocation block size of HFS+ : 0= auto , 512, or 2048
+     */
+    int hfsp_block_size;
+
+    /* Block size of and in APM : 0= auto , 512, or 2048
+     */
+    int apm_block_size;
+
 };
 
 typedef struct ecma119_image Ecma119Image;
 typedef struct ecma119_node Ecma119Node;
 typedef struct joliet_node JolietNode;
 typedef struct iso1999_node Iso1999Node;
+typedef struct hfsplus_node HFSPlusNode;
 typedef struct Iso_File_Src IsoFileSrc;
 typedef struct Iso_Image_Writer IsoImageWriter;
 
@@ -467,6 +511,8 @@ struct ecma119_image
     unsigned int joliet :1;
     unsigned int eltorito :1;
     unsigned int iso1999 :1;
+    unsigned int hfsplus :1;
+    unsigned int fat :1;
 
     unsigned int hardlinks:1; /* see iso_write_opts_set_hardlinks() */
 
@@ -544,6 +590,11 @@ struct ecma119_image
     char *input_charset;
     char *output_charset;
 
+    /* See iso_write_opts and iso_write_opts_set_hfsp_serial_number().
+     * 00...00 means that it shall be generated by libisofs.
+     */
+    uint8_t hfsp_serial_number[8];
+
     unsigned int appendable : 1;
     uint32_t ms_block; /**< start block for a ms image */
     time_t now; /**< Time at which writing began. */
@@ -592,6 +643,29 @@ struct ecma119_image
     uint32_t joliet_l_path_table_pos;
     uint32_t joliet_m_path_table_pos;
 
+    /*
+     * HFS+ related information
+     * (by Vladimir Serbinenko, see libisofs/hfsplus.c)
+     */
+    HFSPlusNode *hfsp_leafs; 
+    struct hfsplus_btree_level *hfsp_levels;
+    uint32_t hfsp_nlevels; 
+    uint32_t hfsp_part_start;
+    uint32_t hfsp_nfiles;
+    uint32_t hfsp_ndirs;
+    uint32_t hfsp_cat_id;
+    uint32_t hfsp_allocation_blocks;
+    uint32_t hfsp_allocation_file_start;
+    uint32_t hfsp_extent_file_start;
+    uint32_t hfsp_catalog_file_start;
+    uint32_t hfsp_total_blocks;
+    uint32_t hfsp_allocation_size;
+    uint32_t hfsp_nleafs;
+    uint32_t hfsp_curleaf;
+    uint32_t hfsp_nnodes;
+    uint32_t hfsp_bless_id[ISO_HFSPLUS_BLESS_MAX];
+    uint32_t hfsp_collision_count;
+
     /*
      * ISO 9660:1999 related information
      */
@@ -751,6 +825,84 @@ struct ecma119_image
 
     char ascii_disc_label[ISO_DISC_LABEL_SIZE];
 
+    /* See IsoImage and libisofs.h */
+    IsoNode *hfsplus_blessed[ISO_HFSPLUS_BLESS_MAX];
+
+    /* Block sizes come from write options.
+       Only change a block size if it is 0. Set only to 512 or 2048.
+       If it stays 0 then it will become 512 or 2048 in time.
+    */
+    /* Blocksize of Apple Partition Map
+       May be defined to 512 or 2048 before writer thread starts.
+    */
+    int apm_block_size;
+
+    /* Allocation block size of HFS+
+       May be defined to 512 or 2048 before hfsplus_writer_create().
+    */
+    int hfsp_block_size;
+    int hfsp_cat_node_size; /* 2 * apm_block_size */
+    int hfsp_iso_block_fac; /* 2048 / apm_block_size */
+
+    /* Apple Partition Map description. To be composed during IsoImageWriter
+       method ->compute_data_blocks() by calling iso_register_apm_entry().
+       Make sure that the composing writers get registered before the
+       gpt_tail_writer.
+    */
+    struct iso_apm_partition_request *apm_req[ISO_APM_ENTRIES_MAX];
+    int apm_req_count;
+    /* bit1= Do not fill gaps in Apple Partition Map 
+       bit2= apm_req entries use apm_block_size in start_block and block_count.
+             Normally these two parameters are counted in 2 KiB blocks.
+    */
+    int apm_req_flags;
+
+    /* MBR partition table description. To be composed during IsoImageWriter
+       method ->compute_data_blocks() by calling iso_register_mbr_entry().
+    */
+    struct iso_mbr_partition_request *mbr_req[ISO_MBR_ENTRIES_MAX];
+    int mbr_req_count;
+
+    char *prep_partition; 
+    uint32_t prep_part_size;
+
+    /* GPT description. To be composed during IsoImageWriter
+       method ->compute_data_blocks() by calling iso_register_gpt_entry().
+       Make sure that the composing writers get registered before the
+       gpt_tail_writer.
+    */
+    struct iso_gpt_partition_request *gpt_req[ISO_GPT_ENTRIES_MAX];
+    int gpt_req_count;
+    /* bit0= GPT partitions may overlap */
+    int gpt_req_flags;
+
+    char *efi_boot_partition;
+    uint32_t efi_boot_part_size;
+    IsoFileSrc *efi_boot_part_filesrc; /* Just a pointer. Do not free. */
+
+    /* Messages from gpt_tail_writer_compute_data_blocks() to
+       iso_write_system_area().
+    */
+    /* Start of GPT entries in System Area, block size 512 */
+    uint32_t gpt_part_start;
+    /* The ISO block number after the backup GPT header , block size 2048 */
+    uint32_t gpt_backup_end; 
+    uint32_t gpt_backup_size;
+    uint32_t gpt_max_entries;
+    int gpt_is_computed;
+
+    /* Message from write_head_part1()/iso_write_system_area() to the
+       write_data() methods of the writers.
+    */
+    uint8_t sys_area_as_written[16 * BLOCK_SIZE];
+
+    /* Size of the filesrc_writer area (data file content).
+       This is available before any IsoImageWriter.compute_data_blocks()
+       is called.
+    */
+    uint32_t filesrc_start;
+    uint32_t filesrc_blocks;
+
 };
 
 #define BP(a,b) [(b) - (a) + 1]
@@ -881,5 +1033,11 @@ struct ecma119_vol_desc_terminator
     uint8_t reserved                 BP(8, 2048);
 };
 
+void ecma119_set_voldescr_times(IsoImageWriter *writer,
+                                struct ecma119_pri_vol_desc *vol);
+
+/* Copies a data file into the ISO image output stream */
+int iso_write_partition_file(Ecma119Image *target, char *path,
+                             uint32_t prepad, uint32_t blocks, int flag);
 
 #endif /*LIBISO_ECMA119_H_*/
