@@ -23,6 +23,11 @@
 /* for gettimeofday() */
 #include <sys/time.h>
 
+#include "filesrc.h"
+#include "ecma119.h"
+#include "eltorito.h"
+#include "system_area.h"
+
 
 /* This code stems from syslinux-3.72/utils/isohybrid, a perl script
 under GPL which is Copyright 2002-2008 H. Peter Anvin.
@@ -45,7 +50,7 @@ license from above stem licenses, typically from LGPL.
 In case its generosity is needed, here is the 2-clause BSD license:
 
 make_isohybrid_mbr.c is copyright 2002-2008 H. Peter Anvin
-                              and 2008-2010 Thomas Schmitt
+                              and 2008-2012 Thomas Schmitt
 
 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
@@ -353,6 +358,12 @@ Main:
 */
 
 
+/* The new stuff about GPT and APM which was learned from Matthew Garret
+   and isohybrid.c is described in doc/boot_sectord.txt chapter
+    "SYSLINUX isohybrid for MBR, UEFI and x86-Mac"
+*/
+
+
 static
 int lba512chs_to_buf(char **wpt, off_t lba, int head_count, int sector_count)
 {
@@ -377,23 +388,202 @@ int lba512chs_to_buf(char **wpt, off_t lba, int head_count, int sector_count)
 }
 
 
+/* Find out whether GPT and APM are desired
+   flag bit0 = register APM and GPT requests in Ecma119Image
+*/
+int assess_isohybrid_gpt_apm(Ecma119Image *t, int *gpt_count, int gpt_idx[128],
+                             int *apm_count, int flag)
+{
+    int i, ilx_opts, j, ret;
+    uint32_t block_count;
+    uint8_t gpt_name[72];
+    static uint8_t zero_uuid[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+    static uint8_t basic_data_uuid[16] = {
+        0xa2, 0xa0, 0xd0, 0xeb, 0xe5, 0xb9, 0x33, 0x44,
+        0x87, 0xc0, 0x68, 0xb6, 0xb7, 0x26, 0x99, 0xc7
+    };
+    static uint8_t hfs_uuid[16] = {
+        0x00, 0x53, 0x46, 0x48, 0x00, 0x00, 0xaa, 0x11,
+        0xaa, 0x11, 0x00, 0x30, 0x65, 0x43, 0xec, 0xac
+    };
+    uint8_t *uuid;
+    static uint64_t gpt_flags = (((uint64_t) 1) << 60) | 1;
+
+    *gpt_count = 0;
+    *apm_count = 0;
+
+    for (i = 0; i < t->catalog->num_bootimages; i++) {
+        ilx_opts = t->catalog->bootimages[i]->isolinux_options;
+        if (((ilx_opts >> 2) & 63) == 1 || ((ilx_opts >> 2) & 63) == 2) {
+            if (*gpt_count < 128)
+                gpt_idx[*gpt_count]= i;
+            (*gpt_count)++;
+            if ((flag & 1) && t->bootsrc[i] != NULL) {
+                /* Register GPT entry */
+                memset(gpt_name, 0, 72);
+                sprintf((char *) gpt_name, "ISOHybrid%d", *gpt_count);
+                iso_ascii_utf_16le(gpt_name);
+                if (((ilx_opts >> 2) & 63) == 2)
+                    uuid = hfs_uuid;
+                else
+                    uuid = basic_data_uuid;
+                block_count = 0;
+                for (j = 0; j < t->bootsrc[i]->nsections; j++)
+                     block_count += t->bootsrc[i]->sections[j].size / 2048;
+                ret = iso_quick_gpt_entry(
+                            t, t->bootsrc[i]->sections[0].block,
+                            block_count, uuid, zero_uuid, gpt_flags,
+                            (uint8_t *) gpt_name);
+                if (ret < 0)
+                    return ret;
+            }
+        }
+        if (ilx_opts & 256) {
+            (*apm_count)++;
+            if ((flag & 1) && t->bootsrc[i] != NULL) {
+                /* Register APM entry */
+                block_count = 0;
+                for (j = 0; j < t->bootsrc[i]->nsections; j++)
+                     block_count += t->bootsrc[i]->sections[j].size / 2048;
+                ret = iso_quick_apm_entry(t, t->bootsrc[i]->sections[0].block,
+                                          block_count, "EFI", "Apple_HFS");
+                if (ret < 0)
+                    return ret;
+                /* Prevent gap filling */
+                t->apm_req_flags |= 2;
+                t->apm_block_size = 2048;
+            }
+        }
+    }
+    if ((flag & 1) && *gpt_count > 0) {
+        /* Register overall GPT partition */
+        memset(gpt_name, 0, 72);
+        sprintf((char *) gpt_name, "ISOHybrid");
+        iso_ascii_utf_16le(gpt_name);
+        /* Let it be open ended. iso_write_gpt() will truncate it as needed. */
+        block_count = 0xffffffff;
+        ret = iso_quick_gpt_entry(t, (uint32_t) 0, block_count,
+                                  basic_data_uuid, zero_uuid, gpt_flags,
+                                  (uint8_t *) gpt_name);
+        if (ret < 0)
+            return ret;
+        /* Remove ban on GPT overlapping */
+        t->gpt_req_flags |= 1;
+    }
+    return ISO_SUCCESS;
+}
+
+
+/* Insert APM head into MBR */
+static int insert_apm_head(uint8_t *buf, int apm_count)
+{
+    int i;
+    static uint8_t apm_mbr_start[32] = {
+        0x33, 0xed, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90,
+        0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90,
+        0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90,
+        0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90
+    };
+    static uint8_t apm_head[32] = {
+        0x45, 0x52, 0x08, 0x00, 0x00, 0x00, 0x90, 0x90,
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
+    };
+
+    if (apm_count) {
+        for (i = 0; i < 32; i++)
+            if(buf[i] != apm_mbr_start[i])
+        break;
+        if (i < 32) {
+            /* Maybe it is already patched by apm_head ? */
+            for (i = 0; i < 32; i++)
+                if(buf[i] != apm_head[i])
+            break;
+        }
+        if (i < 32) {
+            iso_msgs_submit(0,
+               "MBR template file seems not prepared for Apple Partition Map.",
+               0, "FAILURE", 0);
+            return ISO_ISOLINUX_CANT_PATCH;
+        }
+        for (i = 0; i < 32; i++)
+            buf[i] = apm_head[i];
+    }
+    return ISO_SUCCESS;
+}
+
+
+/* Describe GPT boot images as MBR partitions */
+static int gpt_images_as_mbr_partitions(Ecma119Image *t, char *wpt,
+                                        int gpt_idx[128], int *gpt_cursor)
+{
+    int ilx_opts;
+    off_t hd_blocks;
+    static uint8_t dummy_chs[3] = {
+        0xfe, 0xff, 0xff, 
+    };
+
+    wpt[0] = 0;
+    memcpy(wpt + 1, dummy_chs, 3);
+    ilx_opts = t->catalog->bootimages[gpt_idx[*gpt_cursor]]->isolinux_options;
+    if (((ilx_opts >> 2) & 63) == 2)
+        wpt[4] = 0x00; /* HFS gets marked as "Empty" */
+    else
+        ((unsigned char *) wpt)[4] = 0xef; /* "EFI (FAT-12/16/" */
+
+    memcpy(wpt + 5, dummy_chs, 3);
+
+    /* Start LBA (in 512 blocks) */
+    wpt += 8;
+    lsb_to_buf(&wpt, t->bootsrc[gpt_idx[*gpt_cursor]]->sections[0].block * 4,
+               32, 0);
+
+    /* Number of blocks */
+    hd_blocks = t->bootsrc[gpt_idx[*gpt_cursor]]->sections[0].size;
+    hd_blocks = hd_blocks / 512 + !!(hd_blocks % 512);
+    lsb_to_buf(&wpt, (int) hd_blocks, 32, 0);
+
+    (*gpt_cursor)++;
+    return ISO_SUCCESS;
+}
+
+
 /*
  * @param flag  bit0= make own random MBR Id from current time
  */
-int make_isolinux_mbr(int32_t *img_blocks, uint32_t boot_lba,
-                      uint32_t mbr_id, int head_count, int sector_count,
+int make_isolinux_mbr(int32_t *img_blocks, Ecma119Image *t,
                       int part_offset, int part_number, int fs_type,
                       uint8_t *buf, int flag)
 {
     uint32_t id, part, nominal_part_size;
     off_t hd_img_blocks, hd_boot_lba;
     char *wpt;
+    uint32_t boot_lba, mbr_id;
+    int head_count, sector_count, ret;
+    int gpt_count = 0, gpt_idx[128], apm_count = 0, gpt_cursor;
     /* For generating a weak random number */
     struct timeval tv;
     struct timezone tz;
 
     hd_img_blocks = ((off_t) *img_blocks) * (off_t) 4;
 
+    boot_lba = t->bootsrc[0]->sections[0].block;
+    mbr_id = 0;
+    head_count = t->partition_heads_per_cyl;
+    sector_count = t->partition_secs_per_head;
+
+    ret = assess_isohybrid_gpt_apm(t, &gpt_count, gpt_idx, &apm_count, 0);
+    if (ret < 0)
+        return ret;
+
+    /* The rest of APM has already been written by iso_write_apm().
+       But the isohybrid APM head differs from the hfsplus_writer APM head.
+    */
+    ret = insert_apm_head(buf, apm_count);
+    if (ret < 0)
+        return ret;
+
     /* Padding of image_size to a multiple of sector_count*head_count
        happens already at compute time and is implemented by
        an appropriate increase of Ecma119Image->tail_blocks.
@@ -422,10 +612,21 @@ int make_isolinux_mbr(int32_t *img_blocks, uint32_t boot_lba,
 
     /* # Offset 446
     */
+    gpt_cursor= 0;
     for (part = 1 ; part <= 4; part++) {
         if ((int) part != part_number) {
-            /* if this_partition != partition_number: write 16 zero bytes */
+            /* if this_partition != partition_number: write 16 zero bytes
+               (this is now overriden by the eventual desire to announce
+                EFI and HFS boot images.)
+            */
             memset(wpt, 0, 16);
+
+            if (gpt_cursor < gpt_count) {
+                ret = gpt_images_as_mbr_partitions(t, wpt, gpt_idx,
+                                                   &gpt_cursor);
+                if (ret < 0)
+                    return ret;
+            }
             wpt+= 16;
     continue;
         }
