@@ -3,7 +3,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -122,6 +122,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  for(i=0;i<m->rc_filename_count-1;i++)
    strcpy(m->rc_filenames[i],Xorriso_sys_rc_nameS[i]);
  m->rc_filenames[m->rc_filename_count-1][0]= 0;
+ m->arrange_args= 0;
  m->mkisofsrc_done= 0;
 
  m->wdi[0]= 0;
@@ -133,6 +134,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->ino_behavior= 7;
  m->iso_level= 3;
  m->do_joliet= 0;
+ m->do_hfsplus= 0;
+ m->do_fat= 0;
+ m->do_rockridge= 1;
  m->do_iso1999= 0;
  m->do_aaip= 0;
  m->do_md5= 0;
@@ -143,6 +147,8 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->scdbackup_tag_written[0]= 0;
  m->scdbackup_tag_listname[0]= 0;
  m->relax_compliance= 0;
+ m->rr_reloc_dir[0]= 0;
+ m->rr_reloc_flags= 1;
  m->untranslated_name_len= 0;
  m->do_follow_pattern= 1;
  m->do_follow_param= 0;
@@ -172,6 +178,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->do_global_mode= 0;
  m->global_dir_mode= 0555;
  m->global_file_mode= 0444;
+ m->do_tao= 0;
  m->filters= NULL;
  m->filter_list_closed= 0;
  m->zlib_level_default= m->zlib_level= 6;
@@ -189,6 +196,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->displacement_sign= 0;
  m->drives_exclusive= 1;
  m->early_stdio_test= 0;
+ m->cache_num_tiles= 0;
+ m->cache_tile_blocks= 0;
+ m->cache_default= 1 | 2;
  m->do_calm_drive= 1;
  m->indev[0]= 0;
  m->in_drive_handle= NULL;
@@ -247,11 +257,16 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->partition_offset= 0;
  m->partition_secs_per_head= 0;
  m->partition_heads_per_cyl= 0;
+ m->prep_partition[0]= 0;
+ m->efi_boot_partition[0]= 0;
  for(i= 0; i < Xorriso_max_appended_partitionS; i++) {
    m->appended_partitions[i]= NULL;
    m->appended_part_types[i]= 0;
  }
  m->ascii_disc_label[0]= 0;
+ memset(m->hfsp_serial_number, 0, 8);
+ m->hfsp_block_size= 0;
+ m->apm_block_size= 0;
  m->vol_creation_time= 0;
  m->vol_modification_time= 0;
  m->vol_expiration_time= 0;
@@ -284,6 +299,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->disk_exclusions= NULL;
  m->iso_rr_hidings= NULL;
  m->joliet_hidings= NULL;
+ m->hfsplus_hidings= NULL;
  m->disk_excl_mode= 1;
  m->use_stdin= 0;
  m->result_page_length= 0;
@@ -295,6 +311,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
    m->logfile_fp[i]= NULL;
  }
  m->pktlog_fp= NULL;
+ m->stderr_fp= NULL;
  for(i= 0; i < Xorriso_max_outlist_stacK; i++) {
    m->result_msglists[i]= NULL;
    m->info_msglists[i]= NULL;
@@ -406,6 +423,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  if(ret<=0)
    goto failure;
  ret= Exclusions_new(&(m->joliet_hidings), 0);
+ if(ret<=0)
+   goto failure;
+ ret= Exclusions_new(&(m->hfsplus_hidings), 0);
  if(ret<=0)
    goto failure;
  Xorriso_relax_compliance(m, "default", 0);
@@ -474,6 +494,7 @@ int Xorriso_destroy(struct XorrisO **xorriso, int flag)
  Exclusions_destroy(&(m->disk_exclusions), 0);
  Exclusions_destroy(&(m->iso_rr_hidings), 0);
  Exclusions_destroy(&(m->joliet_hidings), 0);
+ Exclusions_destroy(&(m->hfsplus_hidings), 0);
  Xorriso_destroy_all_extf(m, 0);
  Xorriso_lst_destroy_all(&(m->drive_blacklist), 0);
  Xorriso_lst_destroy_all(&(m->drive_greylist), 0);
