@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -225,6 +225,7 @@ int Xorriso_fake_stbuf(struct XorrisO *xorriso, char *path, struct stat *stbuf,
  uint32_t lba;
  char *catcontent = NULL;
  off_t catsize;
+ dev_t dev_number;
 
  memset((char *) stbuf, 0, sizeof(struct stat));
  if(!(flag&1)) {
@@ -256,7 +257,15 @@ int Xorriso_fake_stbuf(struct XorrisO *xorriso, char *path, struct stat *stbuf,
    Xorriso_node_get_dev(xorriso, *node, path, &(stbuf->st_rdev), 0);
  } else if(LIBISO_ISBLK(*node)) {
    stbuf->st_mode|= S_IFBLK;
-   Xorriso_node_get_dev(xorriso, *node, path, &(stbuf->st_rdev), 0);
+   /* ts B11124:
+      Debian mips and mipsel have sizeof(stbuf.st_rdev) == 4
+      whereas sizeof(dev_t) is 8.
+      This workaround assumes that the returned numbers fit into 4 bytes.
+      The may stem from the local filesystem or from the ISO image.
+      At least there will be no memory corruption but only an integer rollover.
+   */ 
+   Xorriso_node_get_dev(xorriso, *node, path, &dev_number, 0);
+   stbuf->st_rdev= dev_number;
  } else if(LIBISO_ISFIFO(*node))
    stbuf->st_mode|= S_IFIFO;
  else if(LIBISO_ISSOCK(*node))
@@ -1302,7 +1311,7 @@ int Xorriso_make_md5(struct XorrisO *xorriso, void *in_node, char *path,
  xorriso->pacifier_count+= size;
  xorriso->pacifier_byte_count+= size;
  Xorriso_pacifier_callback(xorriso, "content bytes read",
-                           xorriso->pacifier_count, 0, "", 0);
+                           xorriso->pacifier_count, 0, "", 8);
  Xorriso_process_msg_queues(xorriso, 0);
  if(ret < 0) {
    Xorriso_report_iso_error(xorriso, "", ret,
@@ -1524,12 +1533,14 @@ int Xorriso_node_eff_hidden(struct XorrisO *xorriso, IsoNode *node, int flag)
  IsoNode *current, *parent;
 
  current= node;
- for(current= node; hidden_state != 3;) {
+ for(current= node; hidden_state != 7;) {
    ret= iso_node_get_hidden(current);
    if(ret & LIBISO_HIDE_ON_RR)
      hidden_state|= 1;
    if(ret & LIBISO_HIDE_ON_JOLIET)
      hidden_state|= 2;
+   if(ret & LIBISO_HIDE_ON_HFSPLUS)
+     hidden_state|= 4;
    parent= (IsoNode *) iso_node_get_parent(current);
    if(parent == current)
  break;
@@ -2058,12 +2069,78 @@ int Xorriso__file_start_lba(IsoNode *node,
 }
 
 
+/* flag bit0= examine sub directories rather than data files
+*/
+int Xorriso_dir_disk_path(struct XorrisO *xorriso, IsoNode *dir_node,
+                          char disk_path[SfileadrL], int flag)
+{
+ int ret;
+ char *npt;
+ IsoNode *node;
+ IsoDir *dir;
+ IsoDirIter *iter= NULL;
+
+ dir= (IsoDir *) dir_node;
+ ret= iso_dir_get_children(dir, &iter);
+ if(ret<0) {
+   Xorriso_cannot_create_iter(xorriso, ret, 0);
+   {ret= -1; goto ex;}
+ }
+ while(1) {
+   ret= iso_dir_iter_next(iter, &node);
+   if(ret < 0) {
+     Xorriso_report_iso_error(xorriso, "", ret,
+               "Error when iterating over directory", 0, "FAILURE", 1);
+     ret= -1; goto ex;
+   }
+   if(ret == 0)
+ break;
+     
+   if(LIBISO_ISDIR(node) && (flag & 1)) {
+     ret= Xorriso_dir_disk_path(xorriso, node, disk_path, flag);
+     if(ret < 0)
+       goto ex;
+     if(ret == 0)
+ continue;
+   } else if(LIBISO_ISREG(node) && !(flag & 1)) {
+     ret= Xorriso_retrieve_disk_path(xorriso, node, disk_path, 0);
+     if(ret < 0)
+       goto ex;
+     if(ret == 0)
+ continue;
+   } else
+ continue;
+   /* Use its parent dir as answer */
+   npt= strrchr(disk_path, '/');
+   if(npt == NULL || npt == disk_path)
+     strcpy(disk_path, "/");
+   else
+     *npt= 0;
+   ret= 1; goto ex;
+ }
+ if(!(flag & 1))
+   ret= Xorriso_dir_disk_path(xorriso, dir_node, disk_path, 1);
+ else
+   ret= 0;
+ex:
+ if(iter != NULL)
+   iso_dir_iter_free(iter);
+ return(ret);
+}
+
+
 int Xorriso_retrieve_disk_path(struct XorrisO *xorriso, IsoNode *node,
                                char disk_path[SfileadrL], int flag)
 {
  IsoFile *file;
  IsoStream *stream= NULL, *input_stream;
  char type_text[80], *source_path = NULL;
+ int ret;
+
+ if(LIBISO_ISDIR(node)) {
+   ret= Xorriso_dir_disk_path(xorriso, node, disk_path, 0);
+   return(ret);
+ }
 
  if(!LIBISO_ISREG(node)) 
    return(0);
