@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -550,7 +550,7 @@ int Xorriso_option_blank(struct XorrisO *xorriso, char *mode, int flag)
    return(0);
  }
  if(xorriso->in_drive_handle == xorriso->out_drive_handle) {
-   if(xorriso->volset_change_pending) {
+   if(Xorriso_change_is_pending(xorriso, 0)) {
      sprintf(xorriso->info_text,
              "%s: Image changes pending. -commit or -rollback first.", cmd);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
@@ -640,9 +640,10 @@ unusable_size:;
 int Xorriso_option_boot_image(struct XorrisO *xorriso, char *form,
                               char *treatment, int flag)
 {
- int was_ok= 1, ret, isolinux_grub= 0, count;
+ int was_ok= 1, ret, isolinux_grub= 0, count, bin_count;
  unsigned int u;
  char *formpt, *treatpt, *eff_path= NULL;
+ uint8_t sn[8];
  double num;
 
  Xorriso_alloc_meM(eff_path, char, SfileadrL);
@@ -666,7 +667,7 @@ cannot_keep_or_patch:;
    if(isolinux_grub)
      goto treatment_patch;
    xorriso->keep_boot_image= 1;
-   xorriso->patch_isolinux_image= 0;
+   xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 0;
    xorriso->boot_image_bin_path[0]= 0;
    xorriso->patch_system_area= 0;
 
@@ -675,7 +676,7 @@ treatment_patch:;
    if(xorriso->boot_count > 0)
      goto cannot_keep_or_patch;
    xorriso->keep_boot_image= 0;
-   xorriso->patch_isolinux_image= 1;
+   xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 1;
    xorriso->boot_image_bin_path[0]= 0;
    if(strcmp(formpt, "grub") == 0) {
      xorriso->patch_isolinux_image|= 2;
@@ -687,7 +688,7 @@ treatment_patch:;
 
  } else if(strcmp(treatpt, "discard")==0) {
    xorriso->keep_boot_image= 0;
-   xorriso->patch_isolinux_image= 0;
+   xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 0;
    xorriso->boot_image_bin_path[0]= 0;
    xorriso->patch_system_area= 0;
    if((xorriso->system_area_options & 0xfc ) == 0)
@@ -765,7 +766,7 @@ treatment_patch:;
      strcat(xorriso->boot_image_cat_path, "boot.cat");
      xorriso->boot_image_load_size= 4 * 512;
      xorriso->keep_boot_image= 0;
-     xorriso->patch_isolinux_image= 1;
+     xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 1;
      strcpy(xorriso->boot_image_bin_form, formpt);
      {ret= 1; goto ex;}
 
@@ -789,7 +790,7 @@ treatment_patch:;
      goto ex;
    xorriso->keep_boot_image= 0;
    if(isolinux_grub) {
-     xorriso->patch_isolinux_image= 1;
+     xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 1;
      if(xorriso->boot_image_bin_path[0])
        xorriso->boot_image_load_size= 4 * 512;
      strcpy(xorriso->boot_image_bin_form, formpt);
@@ -851,9 +852,10 @@ treatment_patch:;
 
  } else if(strncmp(treatpt, "boot_info_table=", 16)==0) {
    if(strcmp(treatpt + 16, "off") == 0)
-     xorriso->patch_isolinux_image= 0;
+     xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 0;
    else if(strcmp(treatpt + 16, "on") == 0)
-     xorriso->patch_isolinux_image= 1 | (2 * (strcmp(treatpt, "grub") == 0));
+     xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) |
+                                    1 | (2 * (strcmp(treatpt, "grub") == 0));
    else
      was_ok= 0;
 
@@ -996,6 +998,71 @@ treatment_patch:;
      {ret= 0; goto ex;}
    }
 
+ } else if(strncmp(treatpt, "hfsplus_serial=", 15) == 0) {
+   ret= Hex_to_bin(treatpt + 15, 8, &bin_count, (unsigned char *) sn, 0);
+   if(ret <= 0 || bin_count != 8) {
+     sprintf(xorriso->info_text,
+             "boot_image %s : Malformed hfsplus_serial : %s",
+             formpt, treatpt + 15);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     Xorriso_msgs_submit(xorriso, 0,
+                         "Expected is a string of 16 hex digits [0-9a-fA-F]",
+                         0, "HINT", 0);
+     ret= 0; goto ex;
+   } else {
+     memcpy(xorriso->hfsp_serial_number, sn, 8);
+   }
+
+ } else if(strncmp(treatpt, "hfsplus_block_size=", 19) == 0) {
+   u= 0;
+   sscanf(treatpt + 19, "%u", &u);
+   if(u != 0 && u!= 512 && u != 2048) {
+     sprintf(xorriso->info_text,
+             "boot_image %s : Malformed hfsplus_block_size : %s",
+             formpt, treatpt + 19);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     Xorriso_msgs_submit(xorriso, 0, "Expected are 0, 512, or 2048",
+                         0, "HINT", 0);
+     ret= 0; goto ex;
+   }
+   xorriso->hfsp_block_size= u;
+
+ } else if(strncmp(treatpt, "apm_block_size=", 15) == 0) {
+   u= 0;
+   sscanf(treatpt + 15, "%u", &u);
+   if(u != 0 && u!= 512 && u != 2048) {
+     sprintf(xorriso->info_text,
+             "boot_image %s : Malformed apm_block_size : %s",
+             formpt, treatpt + 15);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     Xorriso_msgs_submit(xorriso, 0, "Expected are 0, 512, or 2048",
+                         0, "HINT", 0);
+     ret= 0; goto ex;
+   }
+   xorriso->apm_block_size= u;
+
+ } else if(strncmp(treatpt, "efi_boot_part=", 14) == 0) {
+   if(Sfile_str(xorriso->efi_boot_partition, treatpt + 14, 0) <= 0)
+     {ret= -1; goto ex;}
+
+ } else if(strncmp(treatpt, "prep_boot_part=", 15) == 0) {
+   if(Sfile_str(xorriso->prep_partition, treatpt + 15, 0) <= 0)
+     {ret= -1; goto ex;}
+
+ } else if(strncmp(treatpt, "chrp_boot_part=", 15) == 0) {
+   if(strcmp(treatpt + 15, "on") == 0) {
+     xorriso->system_area_options= (xorriso->system_area_options & ~0x3cfc) |
+                                   0x400;
+   } else if(strcmp(treatpt + 15, "off") == 0) {
+     xorriso->system_area_options= xorriso->system_area_options & ~0x3c00;
+   } else {
+     sprintf(xorriso->info_text,
+             "-boot_image %s chrp_boot_part: unknown mode : %s",
+             formpt, treatpt + 15);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+     {ret= 0; goto ex;}
+   }
+
  } else if(strncmp(treatpt, "isohybrid=", 10) == 0 &&
            strcmp(formpt, "isolinux")==0) {
 
@@ -1199,6 +1266,36 @@ ex:;
 }
 
 
+/* Option -changes_pending */
+int Xorriso_option_changes_pending(struct XorrisO *xorriso, char *state,
+                                   int flag)
+{
+ if(strcmp(state, "no") == 0)
+   xorriso->volset_change_pending= 0;
+ else if(strcmp(state, "yes") == 0)
+   xorriso->volset_change_pending= 1;
+ else if(strcmp(state, "mkisofs_printed") == 0)
+   xorriso->volset_change_pending= 2;
+ else if(strcmp(state, "show_status") == 0) {
+   strcpy(xorriso->result_line, "-changes_pending ");
+   if(xorriso->volset_change_pending == 0)
+     strcat(xorriso->result_line, "no");
+   else if(xorriso->volset_change_pending == 2)
+     strcat(xorriso->result_line, "mkisofs_printed");
+   else
+     strcat(xorriso->result_line, "yes");
+   strcat(xorriso->result_line, "\n");
+   Xorriso_result(xorriso,0);
+ } else {
+   sprintf(xorriso->info_text, "-changes_pending: unknown state code '%s'",
+           state);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -charset */
 /* @param flag bit0= set in_charset
                bit1= set out_charset
@@ -1800,7 +1897,7 @@ int Xorriso_option_commit(struct XorrisO *xorriso, int flag)
 {
  int ret;
 
- if(!xorriso->volset_change_pending) {
+ if(!Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text, "-commit: No image modifications pending");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
    {ret= 3; goto ex;}
