@@ -667,15 +667,17 @@ ex:;
                bit2= do not count deleted files with rm and rm_r
                bit3= use Xorriso_findi_sorted() rather than Xorriso_findi()
                      (this can also be ordered by test -sort_lba)
+               bit4= return number of matches plus 1
 */
 int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
                        int *idx, int flag)
 {
  int ret, i, end_idx, type= 0, action, deleter= 0, start_lba, count;
- int list_extattr_head= 0, bsl_mem;
+ int list_extattr_head= 0, bsl_mem, disk_path;
  struct FindjoB *job, *first_job= NULL, *new_job;
  char *start_path, *path= NULL, *cpt, *other_path_start= NULL, *cd_pt;
  char *access_acl_text= NULL, *default_acl_text= NULL, *list_extattr_mode;
+ char *arg1_pt;
 
  struct stat dir_stbuf;
  uid_t user= 0;
@@ -769,14 +771,25 @@ not_enough_arguments:;
      Findjob_set_filter_filter(job, -1, 0);
    } else if(strcmp(argv[i], "-has_md5")==0) {
      Findjob_set_prop_filter(job, 15, 1, 0);
-   } else if(strcmp(argv[i], "-disk_name")==0) {
+   } else if(strcmp(argv[i], "-disk_name")==0 ||
+             strcmp(argv[i], "-disk_path")==0) {
+     disk_path= (strcmp(argv[i], "-disk_path") == 0);
      if(i+1>=end_idx)
        goto not_enough_arguments;
      i++;
-     ret= Findjob_set_name_expr(job, argv[i], 2);
+     arg1_pt= argv[i];
+     if(disk_path) {
+       ret= Xorriso_make_abs_adr(xorriso, xorriso->wdx, argv[i], path,
+                                1 | 2 | 4 | 8);
+       if(ret<=0)
+         goto ex;
+       arg1_pt= path;
+     }
+     ret= Findjob_set_name_expr(job, arg1_pt, 2 + disk_path);
      if(ret<=0) {
        sprintf(xorriso->info_text,
-               "-find[ix]: cannot set -disk_name expression ");
+               "-find[ix]: cannot set %s ",
+               disk_path ? "-disk_path address" : "-disk_name expression");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
        goto sorry_ex;
      }
@@ -796,6 +809,28 @@ not_enough_arguments:;
          goto sorry_ex;
        }
      }
+   } else if(strcmp(argv[i], "-has_hfs_crtp")==0) {
+     if(i + 2 >= end_idx)
+       goto not_enough_arguments;
+     i+= 2;
+     ret= Xorriso_hfsplus_file_creator_type(xorriso, "", NULL,
+                                            argv[i - 1], argv[i], 3);
+     if(ret <= 0)
+       {ret= 0; goto ex;}
+     ret= Findjob_set_crtp_filter(job, argv[i - 1], argv[i], 0);
+     if(ret <= 0) {
+       sprintf(xorriso->info_text, "-findi: cannot setup -has_hfs_crtp test");
+       goto sorry_ex;
+     }
+   } else if(strcmp(argv[i], "-has_hfs_bless")==0) {
+     if(i + 1 >= end_idx)
+       goto not_enough_arguments;
+     i+= 1;
+     ret= Findjob_set_bless_filter(xorriso, job, argv[i], 0);
+     if(ret <= 0) {
+       sprintf(xorriso->info_text, "-findi: cannot setup -has_hfs_bless test");
+       goto sorry_ex;
+     }
    } else if(strcmp(argv[i], "-true") == 0) {
      ret= Findjob_set_false(job, -1, 0);
    } else if(strcmp(argv[i], "-false") == 0) {
@@ -1081,6 +1116,29 @@ not_enough_exec_arguments:;
        Findjob_set_action_target(job, 44, argv[i], 0);
        list_extattr_head= 1;
        list_extattr_mode= argv[i];
+     } else if(strcmp(cpt, "set_hfs_crtp")==0) {
+       if(i + 2 >= end_idx)
+         goto not_enough_exec_arguments;
+       i+= 2;
+       /* Check creator and type for compliance */
+       ret= Xorriso_hfsplus_file_creator_type(xorriso, "", NULL,
+                                              argv[i - 1], argv[i], 1);
+       if(ret <= 0)
+         goto ex;
+       Findjob_set_action_text_2(job, 45, argv[i - 1], argv[i], 0);
+     } else if(strcmp(cpt, "get_hfs_crtp")==0) {
+       Findjob_set_action_target(job, 46, NULL, 0);
+     } else if(strcmp(cpt, "set_hfs_bless")==0) {
+       if(i+1>=end_idx)
+         goto not_enough_exec_arguments;
+       i++;
+       /* Check type of blessing for compliance */
+       ret= Xorriso_hfsplus_bless(xorriso, "", NULL, argv[i], 4);
+       if(ret <= 0)
+         goto ex;
+       Findjob_set_action_target(job, 47, argv[i], 0);
+     } else if(strcmp(cpt, "get_hfs_bless")==0) {
+       Findjob_set_action_target(job, 48, NULL, 0);
      } else {
        sprintf(xorriso->info_text, "-find -exec: unknown action ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
@@ -1155,6 +1213,8 @@ ex:;
    free(access_acl_text);
  if(default_acl_text != NULL)
    free(default_acl_text);
+ if(ret > 0 && (flag & 16) && first_job != NULL)
+   ret= first_job->match_count + 1;
  Findjob_destroy(&first_job, 0);
  Xorriso_free_meM(path);
  Xorriso_free_meM(other_path_start);
@@ -1543,6 +1603,10 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Specifies the name of the Abstract File. (37 chars)",
 "  -joliet \"on\"|\"off\"",
 "                 Generate Joliet info additional to Rock Ridge info.",
+"  -hfsplus \"on\"|\"off\"",
+"                 Generate a HFS+ partition and filesystem within ISO image.",
+"  -rockridge \"on\"|\"off\"",
+"                 Opportunity to omit Rock Ridge info. (Do not do it !)",
 "  -jigdo \"clear\"|\"template_path\"|\"jigdo_path\"|\"md5_path\"",
 "         |\"min_size\"|\"checksum_iso\"|\"checksum_template\"",
 "         |\"compression\"|\"exclude\"|\"demand_md5\"|\"mapping\"",
@@ -1561,9 +1625,12 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              |\"discard\"|\"keep\"|\"patch\"|\"dir=\"|\"bin_path=\"",
 "              |\"cat_path=\"|\"cat_hidden=on|iso_rr|joliet|off\"",
 "              |\"load_size=\"|\"system_area=\"|\"partition_table=on|off\"",
+"              |\"chrp_boot_part=on|off=\"|\"prep_boot_part=\"",
+"              |\"efi_boot_part=\"|\"efi_boot_part=--efi-boot-image\"",
 "              |\"mips_path=\"|\"mipsel_path=\"|\"mips_discard\"",
 "              |\"sparc_label=\"|\"sparc_discard\"",
-"              |\"show_status\"",
+"              |\"hfsplus_serial=\"|\"hfsplus_block_size=\"",
+"              |\"apm_block_size=\"|\"show_status\"",
 "                 Whether to discard or keep an exiting El Torito boot image.",
 "                 ISOLINUX can be made bootable by dir=/ or dir=/isolinux",
 "                 or dir=/boot/isolinux. Others, like GRUB, by bin_path=...",
@@ -1572,6 +1639,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 ISO image by the usual commands like -map or -add.",
 "                 system_area= and partition_table= are for MBR based booting",
 "                 from USB stick. The system_area= file needs not to be added.",
+"                 chrp_boot_part= and prep_boot_part= are for PowerPC.",
+"                 efi_boot_part= is for booting EFI systems from USB stick.",
 "                 mips_path= adds Big Endian MIPS boot files. mipsel_path=",
 "                 sets one Little Endian MIPS boot file. sparc_label=",
 "                 activates SUN Disk Label. All three are mutually exclusive",
@@ -1702,8 +1771,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 access time, modification time, both times.",
 "  -alter_date_r type timestring iso_rr_path [***]",
 "                 Like -alter_date but affecting all files below directories.",
-"  -hide on|iso_rr|joliet|off iso_rr_path [***]",
-"                 Keep names of files out of directory tree, but store their",
+"  -hide on|iso_rr:joliet:hfsplus|off iso_rr_path [***]",
+"                 Keep names of files out of directory trees, but store their",
 "                 data content in the image.",
 "  -find iso_rr_path [test [op] [test ...]] [-exec action [params]]",
 "                 performs an action on files below the given directory in",
@@ -1712,6 +1781,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                   -type b|c|d|p|f|l|s|e, -pending_data, -hidden,",
 "                   -lba_range start count, -damaged, -has_acl, -has_xattr,",
 "                   -has_aaip, -has_filter, -has_md5, -has_any_xattr,",
+"                   -has_hfs_crtp, -has_hfs_bless,",
 "                   -prune, -decision yes|no, -true, -false",
 "                 Operators: -not, -or, -and, -sub, (, -subend, ),",
 "                   -if, -then, -elseif, -else, -endif",
@@ -1720,6 +1790,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                   rm, rm_r, compare, update, report_damage, report_lba,",
 "                   getfacl, setfacl, getfattr, setfattr, get_any_xattr,",
 "                   list_extattr, get_md5, check_md5, make_md5,",
+"                   set_hfs_crtp, get_hfs_crtp, set_hfs_bless, get_hfs_bless,",
 "                   set_filter, show_stream, mkisofs_r, hide, find.",
 "                 params are their parameters except iso_rr_path.",
 "  -mkdir iso_rr_path [...]",
@@ -1804,6 +1875,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -commit_eject  \"in\"|\"out\"|\"all\"|\"none\"",
 "                 Like -commit but rather eject than load image from outdrive.",
 "                 Give up any unejected drive afterwards.",
+"  -write_type \"auto\"|\"tao\"|\"sao/dao\"",
+"                 Set write type for CD-R[W], DVD-R[W], DVD+R, BD-R.",
 "  -close \"on\"|\"off\"",
 "                 If \"on\" then mark the written medium as not appendable.",
 "  -padding number[\"k\"|\"m\"]|\"included\"|\"appended\"",
@@ -2094,6 +2167,22 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 }
 
 
+/* Option -hfsplus "on"|"off" */
+int Xorriso_option_hfsplus(struct XorrisO *xorriso, char *mode, int flag)
+{
+ if(strcmp(mode, "off")==0)
+   xorriso->do_hfsplus= 0;
+ else if(strcmp(mode, "on")==0)
+   xorriso->do_hfsplus= 1;
+ else {
+   sprintf(xorriso->info_text, "-hfsplus: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -hide */
 int Xorriso_option_hide(struct XorrisO *xorriso, char *hide_state,
                         int argc, char **argv, int *idx, int flag)
