@@ -466,19 +466,20 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "dev","dialog","disk_dev_ino","disk_pattern","displacement",
     "dummy","dvd_obs","early_stdio_test", "eject",
     "iso_rr_pattern","follow","format","fs","gid","grow_blindly","hardlinks",
-    "history","indev","in_charset","joliet",
+    "hfsplus","history","indev","in_charset","joliet",
     "list_delimiter","list_extras","list_profiles","local_charset",
     "mark","md5","mount_opts","not_leaf","not_list","not_mgt",
     "options_from_file","osirrox","outdev","out_charset","overwrite",
     "pacifier","padding","path_list","pathspecs","pkt_output",
     "preparer_id","print","print_info","print_mark","prompt",
     "prog","prog_help","publisher","quoted_not_list","quoted_path_list",
-    "reassure","report_about","rom_toc_scan","rr_reloc_dir","scsi_log",
+    "reassure","report_about","rockridge",
+    "rom_toc_scan","rr_reloc_dir","scsi_log",
     "session_log","signal_handling","sleep",
     "speed","split_size","status","status_history_max",
     "stdio_sync","stream_recording","system_id","temp_mem_limit",
     "uid","unregister_filter","use_readline","volid","volset_id",
-    "xattr","zisofs",
+    "write_type","xattr","zisofs",
     ""
  };
  static char arg2_commands[][40]= {
@@ -659,12 +660,13 @@ int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
    "mount",
 
    "* Settings for result writing:",
-   "joliet", "compliance", "rr_reloc_dir", "volid", "volset_id", "publisher",
+   "rockridge", "joliet", "hfsplus","compliance", "rr_reloc_dir",
+   "volid", "volset_id", "publisher",
    "application_id", "system_id", "volume_date", "copyright_file",
    "abstract_file", "biblio_file", "preparer_id", "out_charset",
    "read_mkisofsrc",
    "uid", "gid", "zisofs", "speed", "stream_recording", "dvd_obs",
-   "stdio_sync", "dummy", "fs", "close", "padding",
+   "stdio_sync", "dummy", "fs", "close", "padding", "write_type",
    "grow_blindly", "pacifier", "scdbackup_tag",
 
    "* Bootable ISO images:",
@@ -1270,6 +1272,10 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_hardlinks(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"hfsplus")==0) {
+   (*idx)++;
+   ret= Xorriso_option_hfsplus(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"help")==0) {
    Xorriso_option_help(xorriso,0);
 
@@ -1549,6 +1555,10 @@ next_command:;
  } else if(strcmp(cmd,"rmdir")==0 || strcmp(cmd,"rmdiri")==0) {
    ret= Xorriso_option_rmi(xorriso, argc, argv, idx, 2);
 
+ } else if(strcmp(cmd, "rockridge") == 0) {
+   (*idx)++;
+   ret= Xorriso_option_rockridge(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"rollback")==0) {
    ret= Xorriso_option_rollback(xorriso, 0);
 
@@ -1713,6 +1723,10 @@ next_command:;
    (*idx)+= 2;
    ret= Xorriso_option_volume_date(xorriso, arg1, arg2, 0);
 
+ } else if(strcmp(cmd,"write_type")==0) {
+   (*idx)++;
+   ret= Xorriso_option_write_type(xorriso, arg1, 0);
+
  } else if(strcmp(cmd, "x") == 0) {
    /* only in effect in Xorriso_prescan_args() */;
 
@@ -2529,6 +2543,11 @@ failure:;
    goto failure;
  if(ret > 0)
    hide_attrs|= 2;
+ ret= Exclusions_match(xorriso->hfsplus_hidings, path, 0);
+ if(ret < 0)
+   goto failure;
+ if(ret > 0)
+   hide_attrs|= 4;
  return(hide_attrs);
 }
 
