@@ -544,11 +544,13 @@ ex:;
 */
 int Sfile_bsl_encoder(char **result, char *text, size_t text_len, int flag)
 {
- char *rpt, *wpt;
+ signed char *rpt;
+ char *wpt;
  int count, sq_open= 0, dq_open= 0;
 
  count= 0;
- for(rpt= text; (size_t) (rpt - text) < text_len; rpt++) {
+ for(rpt= (signed char *) text; (size_t) (((char *) rpt) - text) < text_len;
+     rpt++) {
    count++;
    if(flag & 8) {
       if(!(*rpt <= 42 || (*rpt >= 59 && *rpt <= 63) ||
@@ -564,7 +566,8 @@ int Sfile_bsl_encoder(char **result, char *text, size_t text_len, int flag)
  (*result)= wpt= calloc(count + 1, 1);
  if(wpt == NULL)
    return(-1);
- for(rpt= text; (size_t) (rpt - text) < text_len; rpt++) {
+ for(rpt= (signed char *) text; (size_t) (((char *) rpt) - text) < text_len;
+     rpt++) {
    if(*rpt == '\'')
      sq_open= !(sq_open || dq_open);
    if(*rpt == '"')
