@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -433,7 +433,7 @@ int Xorriso_read_lines(struct XorrisO *xorriso, FILE *fp, int *linecount,
      }
  } else {
    (*argv)= Smem_malloC(sizeof(char *));
-   if(argv == NULL)
+   if(*argv == NULL)
      {ret= -1; goto ex;}
    (*argv)[0]= strdup(line);
    if((*argv)[0] == NULL)
@@ -526,7 +526,8 @@ int Xorriso_write_to_channel(struct XorrisO *xorriso,
  bit0= eventually backslash encode linefeeds
  bit1= text is the name of the log file for the given channel 
  bit2= text is the name of the consolidated packet log file for all channels 
-bit15= with bit1 or bit2: close depicted log file
+ bit3= text is the name of the stderr redirection file
+bit15= with bit1 to bit3: close depicted log file
 */
 {
  char *rpt, *npt, *text= NULL;
@@ -559,6 +560,10 @@ bit15= with bit1 or bit2: close depicted log file
    fclose(pktlog_fp);
    xorriso->pktlog_fp= pktlog_fp= NULL;
  }
+ if((flag & 8) && xorriso->stderr_fp != NULL) {
+   fclose(xorriso->stderr_fp);
+   xorriso->stderr_fp= NULL;
+ }
  if(flag&(1<<15))
    {ret= 1; goto ex;}
  if((flag&2)) {
@@ -579,7 +584,13 @@ bit15= with bit1 or bit2: close depicted log file
            Sfile_datestr(time(0),1|2|256));
    fflush(pktlog_fp);
  }
- if(flag&(2|4))
+ if(flag & 8) {
+   truncate(text, (off_t) 0);
+   xorriso->stderr_fp= fopen(text, "a");
+   if(xorriso->stderr_fp == NULL)
+     {ret= 0; goto ex;}
+ }
+ if(flag & (2| 4 | 8))
    {ret= 1; goto ex;}
 
  /* Eventually perform backslash encoding of non-printable characters */
@@ -628,8 +639,13 @@ bit15= with bit1 or bit2: close depicted log file
      printf("%s",text);
      fflush(stdout);
    }
-   if(channel_no==2 || channel_no==3)
-     fprintf(stderr,"%s",text);
+   if(channel_no==2 || channel_no==3) {
+     if(xorriso->stderr_fp != NULL) {
+       fprintf(xorriso->stderr_fp, "%s", text);
+       fflush(xorriso->stderr_fp);
+     } else
+       fprintf(stderr, "%s", text);
+   }
    if(logfile_fp!=NULL) {
      fprintf(logfile_fp,"%s",text);
      fflush(logfile_fp);
@@ -1289,6 +1305,19 @@ int Xorriso_status(struct XorrisO *xorriso, char *filter, FILE *fp, int flag)
  if(!(is_default && no_defaults))
    Xorriso_status_result(xorriso,filter,fp,flag&2);
 
+ is_default= ((xorriso->cache_default & 3) == 3);
+ sprintf(line, "-data_cache_size ");
+ if(xorriso->cache_default & 1)
+   sprintf(line + strlen(line), "default ");
+ else
+   sprintf(line + strlen(line), "%d ", xorriso->cache_num_tiles);
+ if(xorriso->cache_default & 2)
+   sprintf(line + strlen(line), "default\n");
+ else
+   sprintf(line + strlen(line), "%d\n", xorriso->cache_tile_blocks);
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso,filter,fp,flag&2);
+
  is_default= (xorriso->allow_restore==0 && xorriso->do_concat_split==1 &&
               xorriso->do_auto_chmod==0 && xorriso->drives_exclusive == 1);
  mode_pt= "off"; 
@@ -1360,6 +1389,25 @@ int Xorriso_status(struct XorrisO *xorriso, char *filter, FILE *fp, int flag)
      Xorriso_status_result(xorriso,filter,fp,flag&2);
  }
 
+ ret= ((xorriso->system_area_options & 0x3cfc) == 0x400);
+ is_default= (ret == 0);
+ sprintf(line, "-boot_image any chrp_boot_part=%s\n",
+         ret == 1 ? "on" : "off");
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso, filter, fp, flag & 2);
+
+ is_default= (xorriso->prep_partition[0] == 0);
+ sprintf(line,"-boot_image any prep_boot_part=%s\n", 
+              Text_shellsafe(xorriso->prep_partition, sfe, 0));
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso, filter, fp, flag & 2);
+ 
+ is_default= (xorriso->efi_boot_partition[0] == 0);
+ sprintf(line,"-boot_image any efi_boot_part=%s\n",
+              Text_shellsafe(xorriso->efi_boot_partition, sfe, 0));
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso, filter, fp, flag & 2);
+
 #ifdef Xorriso_with_isohybriD
  if(strcmp(form, "isolinux") == 0) {
    static char modes[4][6]= {"off", "auto", "on", "force"};
@@ -1371,6 +1419,30 @@ int Xorriso_status(struct XorrisO *xorriso, char *filter, FILE *fp, int flag)
  }
 #endif /* Xorriso_with_isohybriD */
 
+ is_default= 1;
+ for(i= 0; i < 8; i++)
+   if(xorriso->hfsp_serial_number[i])
+     is_default= 0;
+ sprintf(line, "-boot_image any hfsplus_serial=");
+ for(i= 0; i < 8; i++)
+   sprintf(line + strlen(line), "%-2.2X",
+           (unsigned int) xorriso->hfsp_serial_number[i]);
+ strcat(line, "\n");
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso, filter, fp, flag & 2);
+
+ is_default= (xorriso->hfsp_block_size == 0);
+ sprintf(line, "-boot_image any hfsplus_block_size=%d\n",
+               xorriso->hfsp_block_size);
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso, filter, fp, flag & 2);
+
+ is_default= (xorriso->apm_block_size == 0);
+ sprintf(line, "-boot_image any apm_block_size=%d\n",
+               xorriso->apm_block_size);
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso, filter, fp, flag & 2);
+
  sprintf(line,"-cd %s\n",
          (xorriso->wdi[0] ? Text_shellsafe(xorriso->wdi,sfe,0) : "'/'"));
  Xorriso_status_result(xorriso,filter,fp,flag&2);
@@ -1539,6 +1611,16 @@ int Xorriso_status(struct XorrisO *xorriso, char *filter, FILE *fp, int flag)
  if(!(is_default && no_defaults))
    Xorriso_status_result(xorriso,filter,fp,flag&2);
 
+ is_default= (xorriso->do_rockridge == 1);
+ sprintf(line, "-rockridge %s\n", (xorriso->do_rockridge == 1 ? "on" : "off"));
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso,filter,fp,flag&2);
+
+ is_default= (xorriso->do_hfsplus == 0);
+ sprintf(line,"-hfsplus %s\n", (xorriso->do_hfsplus == 1 ? "on" : "off"));
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso,filter,fp,flag&2);
+
  Xorriso_lst_get_last(xorriso->jigdo_params, &plst, 0);
  Xorriso_lst_get_last(xorriso->jigdo_values, &vlst, 0);
  if(plst == NULL || vlst == NULL) {
@@ -1610,6 +1692,12 @@ int Xorriso_status(struct XorrisO *xorriso, char *filter, FILE *fp, int flag)
  if(!(is_default && no_defaults))
    Xorriso_status_result(xorriso,filter,fp,flag&2);
 
+ is_default= (xorriso->do_tao == 0);
+ sprintf(line,"-write_type %s\n",
+      xorriso->do_tao == 0 ? "auto" : xorriso->do_tao > 0 ? "tao" : "sao/dao");
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso,filter,fp,flag&2);
+
  is_default= !xorriso->do_dummy;
  sprintf(line,"-dummy %s\n",(xorriso->do_dummy ? "on" : "off"));
  if(!(is_default && no_defaults))
@@ -1881,6 +1969,13 @@ int Xorriso_status(struct XorrisO *xorriso, char *filter, FILE *fp, int flag)
  if(!(is_default && no_defaults))
    Xorriso_status_result(xorriso,filter,fp,flag&2);
 
+ is_default= (xorriso->rr_reloc_dir[0] == 0);
+ sprintf(line, "-rr_reloc_dir ");
+ Text_shellsafe(xorriso->rr_reloc_dir, line, 1);
+ strcat(line, "\n");
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso,filter,fp,flag&2);
+
  is_default= (xorriso->assert_volid[0] == 0);
  sprintf(line, "-assert_volid ");
  Text_shellsafe(xorriso->assert_volid, line, 1);
@@ -2011,6 +2106,7 @@ int Xorriso_pacifier_reset(struct XorrisO *xorriso, int flag)
                 bit4= with bit3: count is in blocks, else in bytes
                 bit5= with bit3: report total speed
                 bit6= report with carriage return rather than line feed
+                bit7= with bit5: speed unit for outdev rather than indev
 */
 int Xorriso_pacifier_callback(struct XorrisO *xorriso, char *what_done,
                               off_t count, off_t todo, char *current_object,
@@ -2019,6 +2115,7 @@ int Xorriso_pacifier_callback(struct XorrisO *xorriso, char *what_done,
  double current_time, since, interval_time, speed, speed_factor;
  char count_text[80], byte_text[80], profile_name[80], *speed_unit;
  int ret, profile_number;
+ off_t amount;
  
  current_time= Sfile_microtime(0);
  interval_time= current_time - xorriso->last_update_time;
@@ -2055,19 +2152,24 @@ int Xorriso_pacifier_callback(struct XorrisO *xorriso, char *what_done,
            (flag & (8 | 64)) ? "s" : "seconds");
  }
  speed= -1.0;
+ if(flag & 4)
+   amount= xorriso->pacifier_byte_count;
+ else
+   amount= count;
  if((flag & 8)) {
    if(flag & 32) {
      if(since > 0)
-       speed= count / since;
-   } else if(count >= xorriso->pacifier_prev_count) {
+       speed= amount / since;
+   } else if(amount >= xorriso->pacifier_prev_count) {
      if(interval_time > 0)
-       speed= (count - xorriso->pacifier_prev_count) / interval_time;
+       speed= (amount - xorriso->pacifier_prev_count) / interval_time;
    }
  }
  if(speed >= 0.0) {
    if(flag & 16)
      speed*= 2048.0;
-   ret= Xorriso_get_profile(xorriso, &profile_number, profile_name, 0);
+   ret= Xorriso_get_profile(xorriso, &profile_number, profile_name,
+                            (flag >> 6) & 2);
    speed_factor= 1385000;
    speed_unit= "D";
    if(ret == 2) {
@@ -2080,7 +2182,7 @@ int Xorriso_pacifier_callback(struct XorrisO *xorriso, char *what_done,
    sprintf(xorriso->info_text+strlen(xorriso->info_text), " %s %.1fx%s",
            (flag & 32 ? "=" : ","), speed / speed_factor, speed_unit);
  }
- xorriso->pacifier_prev_count= count;
+ xorriso->pacifier_prev_count= amount;
  if(current_object[0]!=0)
    sprintf(xorriso->info_text+strlen(xorriso->info_text),
            ", now at %s", current_object);
