@@ -494,7 +494,7 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
  if(xorriso->boot_efi_default) {
    emul= 0;
    platform_id= 0xef;
-   xorriso->patch_isolinux_image= 0;
+   xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 0;
  }
  if(platform_id == 0xef || load_size < 0) {
    ret= Xorriso_iso_lstat(xorriso, bin_path, &stbuf, 2 | 4);
@@ -610,28 +610,30 @@ ex:;
 int Xorriso_write_session(struct XorrisO *xorriso, int flag)
 {
  int ret, relax= 0, i, pacifier_speed= 0, data_lba, ext, is_bootable= 0;
- int freshly_bootable= 0, hide_attr, pad_by_libisofs= 0, signal_mode;
+ int freshly_bootable= 0, hide_attr, pad_by_libisofs= 0, signal_mode, role;
  char *xorriso_id= NULL, *img_id, *sfe= NULL, *out_cs, *part_image;
  struct isoburn_imgen_opts *sopts= NULL;
  struct burn_drive_info *dinfo, *source_dinfo;
  struct burn_drive *drive, *source_drive;
  struct burn_disc *disc= NULL;
- struct burn_write_opts *burn_options;
+ struct burn_write_opts *burn_options= NULL;
  off_t readcounter= 0,writecounter= 0;
  int num_sessions= 0, num_tracks= 0;
  struct burn_session **sessions;
  struct burn_track **tracks;
  enum burn_disc_status s;
+ struct burn_multi_caps *caps= NULL;
  IsoImage *image= NULL;
  IsoNode *root_node;
  int profile_number;
- char *profile_name= NULL;
+ char *profile_name= NULL, *reasons= NULL;
  IsoBoot *bootcat_node;
  uint32_t padding;
 
  Xorriso_alloc_meM(sfe, char, 5 * SfileadrL);
  Xorriso_alloc_meM(xorriso_id, char, 256);
  Xorriso_alloc_meM(profile_name, char, 80);
+ Xorriso_alloc_meM(reasons, char, BURN_REASONS_LEN);
 
  ret= Xorriso_finish_hl_update(xorriso, 0);
  if(ret <= 0)
@@ -641,17 +643,29 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
  if(out_cs == NULL)
    Xorriso_get_local_charset(xorriso, &out_cs, 0);
 
+ ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
+                                "on attempt to write", 2);
+ if(ret<=0)
+   goto ex;
  if(!(flag & 1)) {
    ret= Xorriso_auto_format(xorriso, 0);
    if(ret <=0 )
      {ret= 0; goto ex;}
  }
- ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
-                                "on attempt to write", 2);
- if(ret<=0)
-   goto ex;
 
  s= isoburn_disc_get_status(drive);
+ if (xorriso->do_hfsplus && (
+     (xorriso->grow_blindly_msc2 >= 0 &&
+       xorriso->out_drive_handle != xorriso->in_drive_handle)
+     ||
+     (xorriso->out_drive_handle == xorriso->in_drive_handle && 
+      s != BURN_DISC_BLANK)
+    )) {
+   sprintf(xorriso->info_text,
+           "May not grow ISO image while -hfsplus is enabled");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   {ret= 0; goto ex;}
+ }
  if(xorriso->out_drive_handle == xorriso->in_drive_handle) {
    if(abs(xorriso->displacement_sign) == 1 && xorriso->displacement != 0 &&
       s != BURN_DISC_BLANK) {
@@ -753,6 +767,8 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
        hide_attr|= LIBISO_HIDE_ON_RR;
      if(xorriso->boot_image_cat_hidden & 2)
        hide_attr|= LIBISO_HIDE_ON_JOLIET;
+     if(xorriso->boot_image_cat_hidden & 4)
+       hide_attr|= LIBISO_HIDE_ON_HFSPLUS;
      iso_image_set_boot_catalog_hidden(image, hide_attr);
    } else if(xorriso->patch_isolinux_image & 1) {
      if(is_bootable == 1) {
@@ -808,13 +824,16 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
  }
 
  isoburn_igopt_set_level(sopts, xorriso->iso_level);
- ext= isoburn_igopt_rockridge |
+ ext= ((!!xorriso->do_rockridge) * isoburn_igopt_rockridge) |
       ((!!xorriso->do_joliet) * isoburn_igopt_joliet) |
+      ((!!xorriso->do_hfsplus) * isoburn_igopt_hfsplus) |
+      ((!!xorriso->do_fat) * isoburn_igopt_fat) |
       ((!!xorriso->do_iso1999) * isoburn_igopt_iso1999) |
       (( !(xorriso->ino_behavior & 2)) * isoburn_igopt_hardlinks) |
       (( (!(xorriso->ino_behavior & 2)) ||
          (xorriso->do_aaip & (2 | 8 | 16 | 256)) ||
-         (xorriso->do_md5 & (2 | 4))
+         (xorriso->do_md5 & (2 | 4)) ||
+         xorriso->do_hfsplus
        ) * isoburn_igopt_aaip) |
       ((!!(xorriso->do_md5 & 2)) * isoburn_igopt_session_md5) |
       ((!!(xorriso->do_md5 & 4)) * isoburn_igopt_file_md5) |
@@ -825,6 +844,10 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
    ext|= isoburn_igopt_no_emul_toc;
  isoburn_igopt_set_extensions(sopts, ext);
  isoburn_igopt_set_relaxed(sopts, relax);
+ ret = isoburn_igopt_set_rr_reloc(sopts, xorriso->rr_reloc_dir,
+                                  xorriso->rr_reloc_flags);
+ if(ret <= 0)
+   {ret= 0; goto ex;}
  ret= isoburn_igopt_set_untranslated_name_len(sopts,
                                               xorriso->untranslated_name_len);
  if(ret <= 0)
@@ -841,6 +864,16 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
  isoburn_igopt_set_scdbackup_tag(sopts, xorriso->scdbackup_tag_name,
                                  xorriso->scdbackup_tag_time,
                                  xorriso->scdbackup_tag_written);
+ if(xorriso->prep_partition[0]) {
+   ret= isoburn_igopt_set_prep_partition(sopts, xorriso->prep_partition, 0);
+   if(ret <= 0)
+     {ret= 0; goto ex;}
+ }
+ if(xorriso->efi_boot_partition[0]) {
+   ret= isoburn_igopt_set_efi_bootp(sopts, xorriso->efi_boot_partition, 0);
+   if(ret <= 0)
+     {ret= 0; goto ex;}
+ }
  for(i= 0; i < Xorriso_max_appended_partitionS; i++) {
    if(xorriso->appended_partitions[i] == NULL)
  continue;
@@ -854,6 +887,9 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
                                   xorriso->appended_part_types[i], part_image);
  }
  isoburn_igopt_set_disc_label(sopts, xorriso->ascii_disc_label); 
+ isoburn_igopt_set_hfsp_serial_number(sopts, xorriso->hfsp_serial_number);
+ isoburn_igopt_set_hfsp_block_size(sopts, xorriso->hfsp_block_size,
+                                          xorriso->apm_block_size);
 
  if(image!=NULL && 12+strlen(Xorriso_timestamP)<80) {
    strcpy(xorriso_id, xorriso->preparer_id);
@@ -940,6 +976,10 @@ fprintf(stderr, "XORRISO_DEBUG: isoburn_igopt_set_tail_blocks(%d)\n",
      {ret= 0; goto ex;}
  }
 
+ ret = isoburn_igopt_set_write_type(sopts, xorriso->do_tao);
+ if(ret <= 0)
+   goto ex;
+
  Xorriso_set_abort_severity(xorriso, 1);
  if (xorriso->grow_blindly_msc2 >= 0 &&
      xorriso->out_drive_handle != xorriso->in_drive_handle) {
@@ -989,11 +1029,12 @@ fprintf(stderr, "XORRISO_DEBUG: isoburn_igopt_set_tail_blocks(%d)\n",
  if(ret <= 0)
    goto cancel_iso;
 
+ role= burn_drive_get_drive_role(drive);
+
  /* Important: do not return until burn_is_aborting() was checked */
 
  signal_mode= 1;
- ret= burn_drive_get_drive_role(drive);
- if(ret == 1)
+ if(role == 1)
    signal_mode|= 2;
  Xorriso_set_signal_handling(xorriso, signal_mode);
 
@@ -1005,6 +1046,7 @@ fprintf(stderr, "XORRISO_DEBUG: isoburn_igopt_set_tail_blocks(%d)\n",
  xorriso->run_state= 1; /* Indicate that burning has started */
  isoburn_disc_write(burn_options, disc);
  burn_write_opts_free(burn_options);
+ burn_options= NULL;
 
  ret= Xorriso_pacifier_loop(xorriso, drive, pacifier_speed << 4);
  if(burn_is_aborting(0))
@@ -1069,11 +1111,16 @@ ex:;
  if(image != NULL)
    iso_image_unref(image);
  isoburn_igopt_destroy(&sopts, 0);
+ if(burn_options != NULL)
+   burn_write_opts_free(burn_options);
+ if(caps != NULL)
+   burn_disc_free_multi_caps(&caps);
  Xorriso_process_msg_queues(xorriso,0);
  Xorriso_append_scdbackup_record(xorriso, 0);
  Xorriso_free_meM(sfe);
  Xorriso_free_meM(xorriso_id);
  Xorriso_free_meM(profile_name);
+ Xorriso_free_meM(reasons);
  return(ret);
 
 cancel_iso:;
@@ -1886,14 +1933,37 @@ int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
    burn_write_opts_set_start_byte(burn_options, ((off_t) nwa) * (off_t) 2048);
  }
 
- if(burn_write_opts_auto_write_type(burn_options, disc, reasons, 0) ==
-    BURN_WRITE_NONE) {
-   sprintf(xorriso->info_text,
-           "Failed to find a suitable write mode with this media.\n");
-   sprintf(xorriso->info_text+strlen(xorriso->info_text),
-           "Reasons given:\n%s", reasons);
-   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   ret= 0; goto ex;
+ if(xorriso->do_tao) {
+   if (xorriso->do_tao > 0)
+     burn_write_opts_set_write_type(burn_options,
+                                    BURN_WRITE_TAO, BURN_BLOCK_MODE1);
+   else
+     burn_write_opts_set_write_type(burn_options,
+                                    BURN_WRITE_SAO, BURN_BLOCK_SAO);
+                                    
+   ret = burn_precheck_write(burn_options, disc, reasons, 0);
+   if(ret<=0) {
+     sprintf(xorriso->info_text,
+             "Cannot set write type %s for this medium.\n",
+             xorriso->do_tao > 0 ? "TAO" : "SAO");
+     sprintf(xorriso->info_text+strlen(xorriso->info_text),
+             "Reasons given:\n%s", reasons);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
+   sprintf(xorriso->info_text, "Explicitly chosen write type: %s",
+           xorriso->do_tao > 0 ? "TAO" : "SAO");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+ } else {
+   if(burn_write_opts_auto_write_type(burn_options, disc, reasons, 0) ==
+      BURN_WRITE_NONE) {
+     sprintf(xorriso->info_text,
+             "Failed to find a suitable write mode with this media.\n");
+     sprintf(xorriso->info_text+strlen(xorriso->info_text),
+             "Reasons given:\n%s", reasons);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
  }
 
  ret= Xorriso_sanitize_image_size(xorriso, drive, disc, burn_options, 2);
@@ -2064,6 +2134,12 @@ int Xorriso_relax_compliance(struct XorrisO *xorriso, char *mode,
    } else if(l == 14 && strncmp(cpt, "full_ascii_off", l) == 0) {
      xorriso->relax_compliance&= ~isoburn_igopt_allow_full_ascii;
 
+   } else if((l == 10 && strncmp(cpt, "7bit_ascii", l) == 0) ||
+             (l == 13 && strncmp(cpt, "7bit_ascii_on", l) == 0)) {
+     xorriso->relax_compliance|= isoburn_igopt_allow_7bit_ascii;
+   } else if(l == 14 && strncmp(cpt, "7bit_ascii_off", l) == 0) {
+     xorriso->relax_compliance&= ~isoburn_igopt_allow_7bit_ascii;
+
    } else if((l == 17 && strncmp(cpt, "joliet_long_paths", l) == 0) ||
              (l == 20 && strncmp(cpt, "joliet_long_paths_on", l) == 0)) {
      xorriso->relax_compliance|= isoburn_igopt_joliet_longer_paths;
@@ -2218,6 +2294,8 @@ int Xorriso_get_relax_text(struct XorrisO *xorriso, char mode[1024],
    strcat(mode, ":lowercase");
  if(r & isoburn_igopt_allow_full_ascii)
    strcat(mode, ":full_ascii");
+ else if(r & isoburn_igopt_allow_7bit_ascii)
+   strcat(mode, ":7bit_ascii");
  if(r & isoburn_igopt_joliet_longer_paths)
    strcat(mode, ":joliet_long_paths");
  if(r & isoburn_igopt_joliet_long_names)
@@ -2274,7 +2352,9 @@ int Xorriso_set_isolinux_options(struct XorrisO *xorriso,
                             "FATAL", 1);
    ret= -1; goto ex;
  }
- patch_table= xorriso->patch_isolinux_image & 1;
+
+ /* bit0 : 1=boot-info-table , bit2-7 : 1=EFI , 2=HFS+ , bit8 : 1=APM */
+ patch_table = xorriso->patch_isolinux_image & 0x1fd;
  if((flag & 1) && num_boots > 1) {
    ret= el_torito_set_isolinux_options(boots[num_boots - 1], patch_table, 0);
    ret= (ret == 1); goto ex;
@@ -2283,13 +2363,13 @@ int Xorriso_set_isolinux_options(struct XorrisO *xorriso,
  /* Handle patching of first attached boot image or of imported boot images 
  */
  for(i= 0; i < num_boots; i++) {
-   patch_table = xorriso->patch_isolinux_image & 1;
+   patch_table = xorriso->patch_isolinux_image & 0x1fd;
    if(patch_table && !(flag & 1)) {
      if(!el_torito_seems_boot_info_table(boots[i], 0))
-       patch_table= 0;
+       patch_table&= ~1;
      else if((xorriso->patch_isolinux_image & 2) &&
              el_torito_get_boot_platform_id(boots[i]) == 0xef)
-       patch_table= 0;
+       patch_table&= ~1;
    }
    if(i > 0 || xorriso->boot_image_isohybrid == 0) {
      ret= el_torito_set_isolinux_options(boots[i], patch_table, 0);
@@ -2528,7 +2608,7 @@ int Xorriso_close_damaged(struct XorrisO *xorriso, int flag)
  struct burn_drive *drive;
  struct burn_write_opts *burn_options= NULL;
 
- if(xorriso->volset_change_pending) {
+ if(Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text,
            "Image changes pending. -commit or -rollback first");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
