@@ -61,7 +61,7 @@ struct XorrisO;
 */
 #define Xorriso_header_version_majoR  1
 #define Xorriso_header_version_minoR  2
-#define Xorriso_header_version_micrO  2
+#define Xorriso_header_version_micrO  4
 
 
 /** If needed: Something like ".pl01" to indicate a bug fix. Normally empty.
@@ -930,6 +930,9 @@ int Xorriso_option_hardlinks(struct XorrisO *xorriso, char *mode, int flag);
 /* Command -help and part of -prog_help */
 int Xorriso_option_help(struct XorrisO *xorriso, int flag);
 
+/* Option -hfsplus "on"|"off" */
+int Xorriso_option_hfsplus(struct XorrisO *xorriso, char *mode, int flag);
+
 /* Command -hide */
 /* @since 0.6.0 */
 int Xorriso_option_hide(struct XorrisO *xorriso, char *hide_state,
@@ -1053,6 +1056,8 @@ int Xorriso_option_no_rc(struct XorrisO *xorriso, int flag);
                       @since 0.6.0
                 bit1= add to joliet hide list rather than disk exclusions
                       @since 0.6.0
+                bit2= add to HFS+ hide list rather than disk exclusions
+                      @since 1.2.4
 */
 int Xorriso_option_not_leaf(struct XorrisO *xorriso, char *pattern, int flag);
 
@@ -1069,6 +1074,11 @@ int Xorriso_option_not_mgt(struct XorrisO *xorriso, char *setting, int flag);
                 bit1= add to joliet hide list rather than disk exclusions
                       @since 0.6.0
                 bit2= enable disk pattern expansion regardless of -disk_pattern
+                bit8-13= consolidated hide state bits, duplicating bit0-1
+                         @since 1.2.4
+                   bit8= add to iso_rr_hidings, same as bit0
+                   bit9= add to joliet_hidings, same as bit1
+                  bit10= add to hfsplus_hidings
 */ 
 int Xorriso_option_not_paths(struct XorrisO *xorriso, int argc, char **argv,
                              int *idx, int flag);
@@ -1165,6 +1175,9 @@ int Xorriso_option_return_with(struct XorrisO *xorriso, char *severity,
 int Xorriso_option_rmi(struct XorrisO *xorriso, int argc, char **argv,
                        int *idx, int flag);
 
+/* Command -rockridge "on"|"off" */
+int Xorriso_option_rockridge(struct XorrisO *xorriso, char *mode, int flag);
+
 /* Command -rollback */
 /* @param flag bit0= do not -reassure
    @return <=0 error , 1 success, 2 revoked by -reassure
@@ -1291,6 +1304,9 @@ int Xorriso_option_volset_id(struct XorrisO *xorriso, char *name, int flag);
 int Xorriso_option_volume_date(struct XorrisO *xorriso,
                                char *time_type, char *timestring, int flag);
 
+/* Command -write_type */
+int Xorriso_option_write_type(struct XorrisO *xorriso, char *mode, int flag);
+
 /* There is no Xorriso_option_x() because -x has an effect only in
    Xorriso_prescan_args(). Use the flag bits of Xorriso_interpreter() if
    you want to impose command sorting on your own.
