@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  1
 #define xorriso_libisoburn_req_minor  2
-#define xorriso_libisoburn_req_micro  2
+#define xorriso_libisoburn_req_micro  4
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -598,5 +598,12 @@ int Xorriso_list_extras(struct XorrisO *xorriso, char *mode, int flag);
 int Xorriso_set_data_cache(struct XorrisO *xorriso, void *ropts,
                            int num_tiles, int tile_blocks, int flag);
 
+int Xorriso_hfsplus_file_creator_type(struct XorrisO *xorriso, char *path,
+                                      void *in_node, 
+                                      char *creator, char *hfs_type, int flag); 
+
+int Xorriso_hfsplus_bless(struct XorrisO *xorriso, char *path,
+                          void *in_node, char *blessing, int flag);
+
 #endif /* Xorrisoburn_includeD */
 
